/********************************************************************************
** Form generated from reading UI file 'PhotoLoadErrorDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PHOTOLOADERRORDIALOG_H
#define UI_PHOTOLOADERRORDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PhotoLoadErrorDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *photo;
    QLabel *label;
    QPushButton *pbMatch;
    QPushButton *pbLoad;
    QPushButton *pbBarcode;
    QPushButton *pbIgnore;
    QCheckBox *cbRemember;

    void setupUi(QDialog *PhotoLoadErrorDialog)
    {
        if (PhotoLoadErrorDialog->objectName().isEmpty())
            PhotoLoadErrorDialog->setObjectName("PhotoLoadErrorDialog");
        PhotoLoadErrorDialog->resize(328, 202);
        verticalLayout = new QVBoxLayout(PhotoLoadErrorDialog);
        verticalLayout->setObjectName("verticalLayout");
        photo = new QLabel(PhotoLoadErrorDialog);
        photo->setObjectName("photo");

        verticalLayout->addWidget(photo);

        label = new QLabel(PhotoLoadErrorDialog);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label);

        pbMatch = new QPushButton(PhotoLoadErrorDialog);
        pbMatch->setObjectName("pbMatch");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pbMatch->sizePolicy().hasHeightForWidth());
        pbMatch->setSizePolicy(sizePolicy);
        pbMatch->setCheckable(true);
        pbMatch->setChecked(false);

        verticalLayout->addWidget(pbMatch);

        pbLoad = new QPushButton(PhotoLoadErrorDialog);
        pbLoad->setObjectName("pbLoad");
        sizePolicy.setHeightForWidth(pbLoad->sizePolicy().hasHeightForWidth());
        pbLoad->setSizePolicy(sizePolicy);
        pbLoad->setCheckable(true);

        verticalLayout->addWidget(pbLoad);

        pbBarcode = new QPushButton(PhotoLoadErrorDialog);
        pbBarcode->setObjectName("pbBarcode");
        pbBarcode->setCheckable(true);

        verticalLayout->addWidget(pbBarcode);

        pbIgnore = new QPushButton(PhotoLoadErrorDialog);
        pbIgnore->setObjectName("pbIgnore");
        sizePolicy.setHeightForWidth(pbIgnore->sizePolicy().hasHeightForWidth());
        pbIgnore->setSizePolicy(sizePolicy);
        pbIgnore->setCheckable(true);

        verticalLayout->addWidget(pbIgnore);

        cbRemember = new QCheckBox(PhotoLoadErrorDialog);
        cbRemember->setObjectName("cbRemember");

        verticalLayout->addWidget(cbRemember);


        retranslateUi(PhotoLoadErrorDialog);
        QObject::connect(pbMatch, &QPushButton::clicked, PhotoLoadErrorDialog, qOverload<>(&QDialog::accept));
        QObject::connect(pbLoad, &QPushButton::clicked, PhotoLoadErrorDialog, qOverload<>(&QDialog::accept));
        QObject::connect(pbIgnore, &QPushButton::clicked, PhotoLoadErrorDialog, qOverload<>(&QDialog::accept));
        QObject::connect(pbBarcode, &QPushButton::clicked, PhotoLoadErrorDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(PhotoLoadErrorDialog);
    } // setupUi

    void retranslateUi(QDialog *PhotoLoadErrorDialog)
    {
        PhotoLoadErrorDialog->setWindowTitle(QCoreApplication::translate("PhotoLoadErrorDialog", "No Valid geotag", nullptr));
        photo->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "TextLabel", nullptr));
        label->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "This photo does not contain geo-tagging EXIF data.\n"
"What do you want to do next?", nullptr));
        pbMatch->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "Try to match with a track node", nullptr));
        pbLoad->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "Load without associated node", nullptr));
        pbBarcode->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "Extract info from barcode (Walking Papers)", nullptr));
        pbIgnore->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "Ignore it", nullptr));
        cbRemember->setText(QCoreApplication::translate("PhotoLoadErrorDialog", "Do this for all current photos.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PhotoLoadErrorDialog: public Ui_PhotoLoadErrorDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PHOTOLOADERRORDIALOG_H
