/********************************************************************************
** Form generated from reading UI file 'LayerWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAYERWIDGET_H
#define UI_LAYERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LayerWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QCheckBox *cbVisible;
    QLineEdit *edName;

    void setupUi(QWidget *LayerWidget)
    {
        if (LayerWidget->objectName().isEmpty())
            LayerWidget->setObjectName("LayerWidget");
        LayerWidget->resize(207, 31);
        horizontalLayout = new QHBoxLayout(LayerWidget);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setContentsMargins(2, 2, 2, 2);
        horizontalLayout->setObjectName("horizontalLayout");
        cbVisible = new QCheckBox(LayerWidget);
        cbVisible->setObjectName("cbVisible");

        horizontalLayout->addWidget(cbVisible);

        edName = new QLineEdit(LayerWidget);
        edName->setObjectName("edName");
        edName->setReadOnly(true);

        horizontalLayout->addWidget(edName);


        retranslateUi(LayerWidget);

        QMetaObject::connectSlotsByName(LayerWidget);
    } // setupUi

    void retranslateUi(QWidget *LayerWidget)
    {
        LayerWidget->setWindowTitle(QCoreApplication::translate("LayerWidget", "Form", nullptr));
        cbVisible->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class LayerWidget: public Ui_LayerWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LAYERWIDGET_H
