/********************************************************************************
** Form generated from reading UI file 'MinimumRoadProperties.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MINIMUMROADPROPERTIES_H
#define UI_MINIMUMROADPROPERTIES_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RoadProperties
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *mainMayout;
    QLabel *Id;
    QLabel *label;
    QVBoxLayout *variableLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label_4;
    QSpacerItem *spacerItem;
    QToolButton *SourceTagButton;
    QToolButton *RemoveTagButton;
    QTableView *TagView;

    void setupUi(QWidget *RoadProperties)
    {
        if (RoadProperties->objectName().isEmpty())
            RoadProperties->setObjectName("RoadProperties");
        RoadProperties->resize(220, 362);
        vboxLayout = new QVBoxLayout(RoadProperties);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        mainMayout = new QGridLayout();
#ifndef Q_OS_MAC
        mainMayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        mainMayout->setContentsMargins(0, 0, 0, 0);
#endif
        mainMayout->setObjectName("mainMayout");
        Id = new QLabel(RoadProperties);
        Id->setObjectName("Id");

        mainMayout->addWidget(Id, 0, 1, 1, 1);

        label = new QLabel(RoadProperties);
        label->setObjectName("label");

        mainMayout->addWidget(label, 0, 0, 1, 1);


        vboxLayout->addLayout(mainMayout);

        variableLayout = new QVBoxLayout();
        variableLayout->setObjectName("variableLayout");

        vboxLayout->addLayout(variableLayout);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(4);
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        label_4 = new QLabel(RoadProperties);
        label_4->setObjectName("label_4");

        hboxLayout->addWidget(label_4);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        SourceTagButton = new QToolButton(RoadProperties);
        SourceTagButton->setObjectName("SourceTagButton");
        SourceTagButton->setMinimumSize(QSize(23, 23));

        hboxLayout->addWidget(SourceTagButton);

        RemoveTagButton = new QToolButton(RoadProperties);
        RemoveTagButton->setObjectName("RemoveTagButton");
        RemoveTagButton->setMinimumSize(QSize(23, 23));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/Icons/small-remove.xpm"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        RemoveTagButton->setIcon(icon);

        hboxLayout->addWidget(RemoveTagButton);


        vboxLayout->addLayout(hboxLayout);

        TagView = new QTableView(RoadProperties);
        TagView->setObjectName("TagView");
        TagView->setTabKeyNavigation(false);

        vboxLayout->addWidget(TagView);

        QWidget::setTabOrder(RemoveTagButton, TagView);

        retranslateUi(RoadProperties);

        QMetaObject::connectSlotsByName(RoadProperties);
    } // setupUi

    void retranslateUi(QWidget *RoadProperties)
    {
        RoadProperties->setWindowTitle(QCoreApplication::translate("RoadProperties", "Form", nullptr));
        Id->setText(QCoreApplication::translate("RoadProperties", "xxx", nullptr));
        label->setText(QCoreApplication::translate("RoadProperties", "Id", nullptr));
        label_4->setText(QCoreApplication::translate("RoadProperties", "Tags", nullptr));
#if QT_CONFIG(tooltip)
        SourceTagButton->setToolTip(QCoreApplication::translate("RoadProperties", "Add \"source\" tag", nullptr));
#endif // QT_CONFIG(tooltip)
        SourceTagButton->setText(QCoreApplication::translate("RoadProperties", "S", nullptr));
#if QT_CONFIG(tooltip)
        RemoveTagButton->setToolTip(QCoreApplication::translate("RoadProperties", "Remove selected tag(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        RemoveTagButton->setText(QCoreApplication::translate("RoadProperties", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RoadProperties: public Ui_RoadProperties {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MINIMUMROADPROPERTIES_H
