/********************************************************************************
** Form generated from reading UI file 'DirtyDock.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DIRTYDOCK_H
#define UI_DIRTYDOCK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DirtyDockWidget
{
public:
    QVBoxLayout *vboxLayout;
    QFrame *frame;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QPushButton *pbCleanupHistory;
    QListWidget *ChangesList;

    void setupUi(QWidget *DirtyDockWidget)
    {
        if (DirtyDockWidget->objectName().isEmpty())
            DirtyDockWidget->setObjectName("DirtyDockWidget");
        DirtyDockWidget->resize(302, 459);
        vboxLayout = new QVBoxLayout(DirtyDockWidget);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        frame = new QFrame(DirtyDockWidget);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame);
        vboxLayout1->setObjectName("vboxLayout1");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(frame);
        label->setObjectName("label");
        label->setWordWrap(true);

        horizontalLayout->addWidget(label);

        pbCleanupHistory = new QPushButton(frame);
        pbCleanupHistory->setObjectName("pbCleanupHistory");
        pbCleanupHistory->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pbCleanupHistory->sizePolicy().hasHeightForWidth());
        pbCleanupHistory->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(pbCleanupHistory);


        vboxLayout1->addLayout(horizontalLayout);

        ChangesList = new QListWidget(frame);
        ChangesList->setObjectName("ChangesList");
        ChangesList->setContextMenuPolicy(Qt::CustomContextMenu);
        ChangesList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout1->addWidget(ChangesList);


        vboxLayout->addWidget(frame);


        retranslateUi(DirtyDockWidget);

        QMetaObject::connectSlotsByName(DirtyDockWidget);
    } // setupUi

    void retranslateUi(QWidget *DirtyDockWidget)
    {
        DirtyDockWidget->setWindowTitle(QCoreApplication::translate("DirtyDockWidget", "Undo", nullptr));
        label->setText(QCoreApplication::translate("DirtyDockWidget", "There are <b>no</b> dirty objects.", nullptr));
        pbCleanupHistory->setText(QCoreApplication::translate("DirtyDockWidget", "Cleanup", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DirtyDockWidget: public Ui_DirtyDockWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DIRTYDOCK_H
