/****************************************************************************
** Meta object code from reading C++ file 'msbingmapadapter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/plugins/background/MMsBingMapBackground/msbingmapadapter.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'msbingmapadapter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16MsBingMapAdapterE_t {};
} // unnamed namespace

template <> constexpr inline auto MsBingMapAdapter::qt_create_metaobjectdata<qt_meta_tag_ZN16MsBingMapAdapterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MsBingMapAdapter",
        "on_adapterDataFinished",
        "",
        "QNetworkReply*",
        "rply"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_adapterDataFinished'
        QtMocHelpers::SlotData<void(QNetworkReply *)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MsBingMapAdapter, qt_meta_tag_ZN16MsBingMapAdapterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MsBingMapAdapter::staticMetaObject = { {
    QMetaObject::SuperData::link<MapAdapter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MsBingMapAdapterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MsBingMapAdapterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MsBingMapAdapterE_t>.metaTypes,
    nullptr
} };

void MsBingMapAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MsBingMapAdapter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_adapterDataFinished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *MsBingMapAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MsBingMapAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MsBingMapAdapterE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IMapWatermark"))
        return static_cast< IMapWatermark*>(this);
    if (!strcmp(_clname, "com.cbsoft.Merkaartor.IMapAdapter/1.12"))
        return static_cast< IMapAdapter*>(this);
    if (!strcmp(_clname, "com.cbsoft.Merkaartor.IMapWatermark/1.0"))
        return static_cast< IMapWatermark*>(this);
    return MapAdapter::qt_metacast(_clname);
}

int MsBingMapAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MapAdapter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto MsBingMapAdapterFactory::qt_create_metaobjectdata<qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MsBingMapAdapterFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MsBingMapAdapterFactory, qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MsBingMapAdapterFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>.metaTypes,
    nullptr
} };

void MsBingMapAdapterFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MsBingMapAdapterFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *MsBingMapAdapterFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MsBingMapAdapterFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23MsBingMapAdapterFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IMapAdapterFactory"))
        return static_cast< IMapAdapterFactory*>(this);
    if (!strcmp(_clname, "com.cbsoft.Merkaartor.IMapAdapterFactory/1.0"))
        return static_cast< IMapAdapterFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int MsBingMapAdapterFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_MsBingMapAdapterFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1e,  'b',  'e',  '.',  'm',  'e', 
    'r',  'k',  'a',  'a',  'r',  't',  'o',  'r', 
    '.',  'm',  's',  'b',  'i',  'n',  'g',  'm', 
    'a',  'p',  'a',  'd',  'a',  'p',  't',  'e', 
    'r', 
    // "className"
    0x03,  0x77,  'M',  's',  'B',  'i',  'n',  'g', 
    'M',  'a',  'p',  'A',  'd',  'a',  'p',  't', 
    'e',  'r',  'F',  'a',  'c',  't',  'o',  'r', 
    'y', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(MsBingMapAdapterFactory, MsBingMapAdapterFactory, qt_pluginMetaDataV2_MsBingMapAdapterFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_MsBingMapAdapterFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x1e,  'b',  'e',  '.',  'm',  'e', 
    'r',  'k',  'a',  'a',  'r',  't',  'o',  'r', 
    '.',  'm',  's',  'b',  'i',  'n',  'g',  'm', 
    'a',  'p',  'a',  'd',  'a',  'p',  't',  'e', 
    'r', 
    // "className"
    0x03,  0x77,  'M',  's',  'B',  'i',  'n',  'g', 
    'M',  'a',  'p',  'A',  'd',  'a',  'p',  't', 
    'e',  'r',  'F',  'a',  'c',  't',  'o',  'r', 
    'y', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(MsBingMapAdapterFactory, MsBingMapAdapterFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
