/********************************************************************************
** Form generated from reading UI file 'FeaturesDock.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FEATURESDOCK_H
#define UI_FEATURESDOCK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FeaturesDockWidget
{
public:
    QVBoxLayout *vboxLayout;
    QFrame *frame;
    QVBoxLayout *vboxLayout1;
    QTabBar *tabBar;
    QListWidget *FeaturesList;
    QHBoxLayout *horizontalLayout;
    QCheckBox *cbWithin;
    QLabel *lblOnlyViewport;
    QSpacerItem *horizontalSpacer;
    QToolButton *btFind;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *cbSelectionFilter;
    QLabel *label;
    QSpacerItem *horizontalSpacer_2;
    QToolButton *btReset;

    void setupUi(QWidget *FeaturesDockWidget)
    {
        if (FeaturesDockWidget->objectName().isEmpty())
            FeaturesDockWidget->setObjectName("FeaturesDockWidget");
        FeaturesDockWidget->resize(292, 481);
        vboxLayout = new QVBoxLayout(FeaturesDockWidget);
        vboxLayout->setSpacing(2);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        frame = new QFrame(FeaturesDockWidget);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame);
        vboxLayout1->setSpacing(2);
        vboxLayout1->setContentsMargins(4, 4, 4, 4);
        vboxLayout1->setObjectName("vboxLayout1");
        tabBar = new QTabBar(frame);
        tabBar->setObjectName("tabBar");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabBar->sizePolicy().hasHeightForWidth());
        tabBar->setSizePolicy(sizePolicy);
        tabBar->setMinimumSize(QSize(1, 0));

        vboxLayout1->addWidget(tabBar);

        FeaturesList = new QListWidget(frame);
        FeaturesList->setObjectName("FeaturesList");
        FeaturesList->setContextMenuPolicy(Qt::CustomContextMenu);
        FeaturesList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout1->addWidget(FeaturesList);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        cbWithin = new QCheckBox(frame);
        cbWithin->setObjectName("cbWithin");

        horizontalLayout->addWidget(cbWithin);

        lblOnlyViewport = new QLabel(frame);
        lblOnlyViewport->setObjectName("lblOnlyViewport");
        lblOnlyViewport->setWordWrap(false);

        horizontalLayout->addWidget(lblOnlyViewport);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        btFind = new QToolButton(frame);
        btFind->setObjectName("btFind");

        horizontalLayout->addWidget(btFind);


        vboxLayout1->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        cbSelectionFilter = new QCheckBox(frame);
        cbSelectionFilter->setObjectName("cbSelectionFilter");

        horizontalLayout_3->addWidget(cbSelectionFilter);

        label = new QLabel(frame);
        label->setObjectName("label");
        label->setWordWrap(false);

        horizontalLayout_3->addWidget(label);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);

        btReset = new QToolButton(frame);
        btReset->setObjectName("btReset");

        horizontalLayout_3->addWidget(btReset);


        vboxLayout1->addLayout(horizontalLayout_3);


        vboxLayout->addWidget(frame);

#if QT_CONFIG(shortcut)
        lblOnlyViewport->setBuddy(cbWithin);
        label->setBuddy(cbSelectionFilter);
#endif // QT_CONFIG(shortcut)

        retranslateUi(FeaturesDockWidget);

        QMetaObject::connectSlotsByName(FeaturesDockWidget);
    } // setupUi

    void retranslateUi(QWidget *FeaturesDockWidget)
    {
        FeaturesDockWidget->setWindowTitle(QCoreApplication::translate("FeaturesDockWidget", "Features", nullptr));
#if QT_CONFIG(tooltip)
        lblOnlyViewport->setToolTip(QCoreApplication::translate("FeaturesDockWidget", "Only features fully within the viewport", nullptr));
#endif // QT_CONFIG(tooltip)
        lblOnlyViewport->setText(QCoreApplication::translate("FeaturesDockWidget", "Filter by viewport", nullptr));
        btFind->setText(QCoreApplication::translate("FeaturesDockWidget", "Find...", nullptr));
        cbSelectionFilter->setText(QString());
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("FeaturesDockWidget", "Only show features intersecting current selection.", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("FeaturesDockWidget", "Filter by selection", nullptr));
        btReset->setText(QCoreApplication::translate("FeaturesDockWidget", "Reset", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FeaturesDockWidget: public Ui_FeaturesDockWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FEATURESDOCK_H
