/****************************************************************************
** Meta object code from reading C++ file 'PropertiesDock.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/Docks/PropertiesDock.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PropertiesDock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14PropertiesDockE_t {};
} // unnamed namespace

template <> constexpr inline auto PropertiesDock::qt_create_metaobjectdata<qt_meta_tag_ZN14PropertiesDockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PropertiesDock",
        "selectionChanged",
        "",
        "on_TrackPointLat_editingFinished",
        "on_TrackPointLon_editingFinished",
        "on_Member_customContextMenuRequested",
        "QPoint",
        "pos",
        "on_Member_clicked",
        "QModelIndex",
        "index",
        "on_Member_selected",
        "on_btMemberUp_clicked",
        "on_btMemberDown_clicked",
        "on_RemoveMemberButton_clicked",
        "on_RemoveTagButton_clicked",
        "on_SourceTagButton_clicked",
        "on_SelectionList_itemSelectionChanged",
        "on_SelectionList_itemDoubleClicked",
        "QListWidgetItem*",
        "item",
        "executePendingSelectionChange",
        "on_SelectionList_customContextMenuRequested",
        "on_centerAction_triggered",
        "on_centerZoomAction_triggered",
        "on_tag_changed",
        "k",
        "v",
        "on_tag_cleared",
        "on_template_changed",
        "TagTemplate*",
        "aNewTemplate",
        "adjustSelection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectionChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TrackPointLat_editingFinished'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TrackPointLon_editingFinished'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_Member_customContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'on_Member_clicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'on_Member_selected'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_btMemberUp_clicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_btMemberDown_clicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_RemoveMemberButton_clicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_RemoveTagButton_clicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_SourceTagButton_clicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_SelectionList_itemSelectionChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_SelectionList_itemDoubleClicked'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'executePendingSelectionChange'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_SelectionList_customContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'on_centerAction_triggered'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_centerZoomAction_triggered'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_tag_changed'
        QtMocHelpers::SlotData<void(QString, QString)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 }, { QMetaType::QString, 27 },
        }}),
        // Slot 'on_tag_cleared'
        QtMocHelpers::SlotData<void(QString)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'on_template_changed'
        QtMocHelpers::SlotData<void(TagTemplate *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'adjustSelection'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PropertiesDock, qt_meta_tag_ZN14PropertiesDockE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PropertiesDock::staticMetaObject = { {
    QMetaObject::SuperData::link<MDockAncestor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PropertiesDockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PropertiesDockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PropertiesDockE_t>.metaTypes,
    nullptr
} };

void PropertiesDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PropertiesDock *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectionChanged(); break;
        case 1: _t->on_TrackPointLat_editingFinished(); break;
        case 2: _t->on_TrackPointLon_editingFinished(); break;
        case 3: _t->on_Member_customContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 4: _t->on_Member_clicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 5: _t->on_Member_selected(); break;
        case 6: _t->on_btMemberUp_clicked(); break;
        case 7: _t->on_btMemberDown_clicked(); break;
        case 8: _t->on_RemoveMemberButton_clicked(); break;
        case 9: _t->on_RemoveTagButton_clicked(); break;
        case 10: _t->on_SourceTagButton_clicked(); break;
        case 11: _t->on_SelectionList_itemSelectionChanged(); break;
        case 12: _t->on_SelectionList_itemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 13: _t->executePendingSelectionChange(); break;
        case 14: _t->on_SelectionList_customContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->on_centerAction_triggered(); break;
        case 16: _t->on_centerZoomAction_triggered(); break;
        case 17: _t->on_tag_changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 18: _t->on_tag_cleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->on_template_changed((*reinterpret_cast<std::add_pointer_t<TagTemplate*>>(_a[1]))); break;
        case 20: _t->adjustSelection(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PropertiesDock::*)()>(_a, &PropertiesDock::selectionChanged, 0))
            return;
    }
}

const QMetaObject *PropertiesDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PropertiesDockE_t>.strings))
        return static_cast<void*>(this);
    return MDockAncestor::qt_metacast(_clname);
}

int PropertiesDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MDockAncestor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void PropertiesDock::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
