/********************************************************************************
** Form generated from reading UI file 'OsmServerWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OSMSERVERWIDGET_H
#define UI_OSMSERVERWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OsmServerWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *rbOsmServerSelected;
    QComboBox *edOsmServerUrl;
    QToolButton *tbOsmServerAdd;
    QToolButton *tbOsmServerDel;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label;
    QComboBox *authType;
    QStackedWidget *credentials;
    QWidget *Page1;
    QHBoxLayout *horizontalLayout;
    QLabel *label_14;
    QLineEdit *edOsmServerUser;
    QLabel *label_13;
    QLineEdit *edOsmServerPwd;
    QWidget *page;
    QHBoxLayout *horizontalLayout_3;
    QLabel *lbLoginState;
    QSpacerItem *horizontalSpacer;
    QPushButton *tbOAuth2Login;

    void setupUi(QWidget *OsmServerWidget)
    {
        if (OsmServerWidget->objectName().isEmpty())
            OsmServerWidget->setObjectName("OsmServerWidget");
        OsmServerWidget->resize(552, 195);
        verticalLayout = new QVBoxLayout(OsmServerWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        rbOsmServerSelected = new QRadioButton(OsmServerWidget);
        rbOsmServerSelected->setObjectName("rbOsmServerSelected");

        horizontalLayout_2->addWidget(rbOsmServerSelected);

        edOsmServerUrl = new QComboBox(OsmServerWidget);
        edOsmServerUrl->addItem(QString());
        edOsmServerUrl->addItem(QString());
        edOsmServerUrl->setObjectName("edOsmServerUrl");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(edOsmServerUrl->sizePolicy().hasHeightForWidth());
        edOsmServerUrl->setSizePolicy(sizePolicy);
        edOsmServerUrl->setCursor(QCursor(Qt::CursorShape::IBeamCursor));
        edOsmServerUrl->setMouseTracking(true);
        edOsmServerUrl->setAcceptDrops(true);
        edOsmServerUrl->setEditable(true);

        horizontalLayout_2->addWidget(edOsmServerUrl);

        tbOsmServerAdd = new QToolButton(OsmServerWidget);
        tbOsmServerAdd->setObjectName("tbOsmServerAdd");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/Icons/actions/list-add.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        tbOsmServerAdd->setIcon(icon);

        horizontalLayout_2->addWidget(tbOsmServerAdd);

        tbOsmServerDel = new QToolButton(OsmServerWidget);
        tbOsmServerDel->setObjectName("tbOsmServerDel");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/Icons/actions/list-remove.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        tbOsmServerDel->setIcon(icon1);

        horizontalLayout_2->addWidget(tbOsmServerDel);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label = new QLabel(OsmServerWidget);
        label->setObjectName("label");

        horizontalLayout_6->addWidget(label);

        authType = new QComboBox(OsmServerWidget);
        authType->addItem(QString());
        authType->addItem(QString());
        authType->addItem(QString());
        authType->setObjectName("authType");

        horizontalLayout_6->addWidget(authType);


        verticalLayout->addLayout(horizontalLayout_6);

        credentials = new QStackedWidget(OsmServerWidget);
        credentials->setObjectName("credentials");
        Page1 = new QWidget();
        Page1->setObjectName("Page1");
        horizontalLayout = new QHBoxLayout(Page1);
        horizontalLayout->setObjectName("horizontalLayout");
        label_14 = new QLabel(Page1);
        label_14->setObjectName("label_14");

        horizontalLayout->addWidget(label_14);

        edOsmServerUser = new QLineEdit(Page1);
        edOsmServerUser->setObjectName("edOsmServerUser");

        horizontalLayout->addWidget(edOsmServerUser);

        label_13 = new QLabel(Page1);
        label_13->setObjectName("label_13");

        horizontalLayout->addWidget(label_13);

        edOsmServerPwd = new QLineEdit(Page1);
        edOsmServerPwd->setObjectName("edOsmServerPwd");
        edOsmServerPwd->setEchoMode(QLineEdit::Password);

        horizontalLayout->addWidget(edOsmServerPwd);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 5);
        horizontalLayout->setStretch(2, 1);
        horizontalLayout->setStretch(3, 3);
        credentials->addWidget(Page1);
        page = new QWidget();
        page->setObjectName("page");
        horizontalLayout_3 = new QHBoxLayout(page);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        lbLoginState = new QLabel(page);
        lbLoginState->setObjectName("lbLoginState");

        horizontalLayout_3->addWidget(lbLoginState);

        horizontalSpacer = new QSpacerItem(309, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);

        tbOAuth2Login = new QPushButton(page);
        tbOAuth2Login->setObjectName("tbOAuth2Login");

        horizontalLayout_3->addWidget(tbOAuth2Login);

        credentials->addWidget(page);

        verticalLayout->addWidget(credentials);


        retranslateUi(OsmServerWidget);
        QObject::connect(authType, &QComboBox::currentIndexChanged, credentials, &QStackedWidget::setCurrentIndex);

        authType->setCurrentIndex(0);
        credentials->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(OsmServerWidget);
    } // setupUi

    void retranslateUi(QWidget *OsmServerWidget)
    {
        OsmServerWidget->setWindowTitle(QCoreApplication::translate("OsmServerWidget", "Form", nullptr));
        rbOsmServerSelected->setText(QCoreApplication::translate("OsmServerWidget", "URL:", nullptr));
        edOsmServerUrl->setItemText(0, QCoreApplication::translate("OsmServerWidget", "https://www.openstreetmap.org/", nullptr));
        edOsmServerUrl->setItemText(1, QCoreApplication::translate("OsmServerWidget", "https://master.apis.dev.openstreetmap.org/", nullptr));

        tbOsmServerAdd->setText(QCoreApplication::translate("OsmServerWidget", "...", nullptr));
        tbOsmServerDel->setText(QCoreApplication::translate("OsmServerWidget", "...", nullptr));
        label->setText(QCoreApplication::translate("OsmServerWidget", "Authentication type", nullptr));
        authType->setItemText(0, QCoreApplication::translate("OsmServerWidget", "Username and password (deprecated)", nullptr));
        authType->setItemText(1, QCoreApplication::translate("OsmServerWidget", "OAuth2 Redirect (redirect via 127.0.0.1:1337)", nullptr));
        authType->setItemText(2, QCoreApplication::translate("OsmServerWidget", "OAuth2 OOB (manually copy code)", nullptr));

        label_14->setText(QCoreApplication::translate("OsmServerWidget", "User:", nullptr));
        label_13->setText(QCoreApplication::translate("OsmServerWidget", "Pwd:", nullptr));
        lbLoginState->setText(QCoreApplication::translate("OsmServerWidget", "Login state: Unknown", nullptr));
        tbOAuth2Login->setText(QCoreApplication::translate("OsmServerWidget", "Login", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OsmServerWidget: public Ui_OsmServerWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OSMSERVERWIDGET_H
