/********************************************************************************
** Form generated from reading UI file 'namefinderwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NAMEFINDERWIDGET_H
#define UI_NAMEFINDERWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_NameFinderWidgetUi
{
public:
    QVBoxLayout *verticalLayout;
    QTableView *tableView;

    void setupUi(QWidget *NameFinderWidgetUi)
    {
        if (NameFinderWidgetUi->objectName().isEmpty())
            NameFinderWidgetUi->setObjectName("NameFinderWidgetUi");
        NameFinderWidgetUi->resize(400, 300);
        verticalLayout = new QVBoxLayout(NameFinderWidgetUi);
        verticalLayout->setObjectName("verticalLayout");
        tableView = new QTableView(NameFinderWidgetUi);
        tableView->setObjectName("tableView");
        tableView->setContextMenuPolicy(Qt::NoContextMenu);
        tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
        tableView->setAlternatingRowColors(true);
        tableView->setSelectionMode(QAbstractItemView::SingleSelection);
        tableView->setSelectionBehavior(QAbstractItemView::SelectRows);

        verticalLayout->addWidget(tableView);


        retranslateUi(NameFinderWidgetUi);

        QMetaObject::connectSlotsByName(NameFinderWidgetUi);
    } // setupUi

    void retranslateUi(QWidget *NameFinderWidgetUi)
    {
        NameFinderWidgetUi->setWindowTitle(QCoreApplication::translate("NameFinderWidgetUi", "Form", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NameFinderWidgetUi: public Ui_NameFinderWidgetUi {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NAMEFINDERWIDGET_H
