/********************************************************************************
** Form generated from reading UI file 'OAuth2OOBDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OAUTH2OOBDIALOG_H
#define UI_OAUTH2OOBDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_OAuth2OOBDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label_3;
    QFormLayout *formLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLineEdit *loginUrl;
    QPushButton *btnCopy;
    QPushButton *btnOpen;
    QLabel *label_2;
    QLineEdit *loginCode;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OAuth2OOBDialog)
    {
        if (OAuth2OOBDialog->objectName().isEmpty())
            OAuth2OOBDialog->setObjectName("OAuth2OOBDialog");
        OAuth2OOBDialog->resize(586, 131);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(OAuth2OOBDialog->sizePolicy().hasHeightForWidth());
        OAuth2OOBDialog->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(OAuth2OOBDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label_3 = new QLabel(OAuth2OOBDialog);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy1);

        verticalLayout_2->addWidget(label_3);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setVerticalSpacing(5);
        label = new QLabel(OAuth2OOBDialog);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        loginUrl = new QLineEdit(OAuth2OOBDialog);
        loginUrl->setObjectName("loginUrl");
        loginUrl->setEnabled(true);
        loginUrl->setReadOnly(true);

        horizontalLayout->addWidget(loginUrl);

        btnCopy = new QPushButton(OAuth2OOBDialog);
        btnCopy->setObjectName("btnCopy");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(btnCopy->sizePolicy().hasHeightForWidth());
        btnCopy->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(btnCopy);

        btnOpen = new QPushButton(OAuth2OOBDialog);
        btnOpen->setObjectName("btnOpen");

        horizontalLayout->addWidget(btnOpen);


        formLayout->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        label_2 = new QLabel(OAuth2OOBDialog);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        loginCode = new QLineEdit(OAuth2OOBDialog);
        loginCode->setObjectName("loginCode");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, loginCode);


        verticalLayout_2->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(OAuth2OOBDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);


        retranslateUi(OAuth2OOBDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, OAuth2OOBDialog, qOverload<>(&QDialog::reject));
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, OAuth2OOBDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(OAuth2OOBDialog);
    } // setupUi

    void retranslateUi(QDialog *OAuth2OOBDialog)
    {
        OAuth2OOBDialog->setWindowTitle(QCoreApplication::translate("OAuth2OOBDialog", "OAuth2 OOB", nullptr));
        label_3->setText(QCoreApplication::translate("OAuth2OOBDialog", "Open the following URL in a browser. After logging in, paste the displayed authentication code below.", nullptr));
        label->setText(QCoreApplication::translate("OAuth2OOBDialog", "Login URL", nullptr));
        btnCopy->setText(QCoreApplication::translate("OAuth2OOBDialog", "Copy", nullptr));
        btnOpen->setText(QCoreApplication::translate("OAuth2OOBDialog", "Open", nullptr));
        label_2->setText(QCoreApplication::translate("OAuth2OOBDialog", "Authentication code", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OAuth2OOBDialog: public Ui_OAuth2OOBDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OAUTH2OOBDIALOG_H
