/********************************************************************************
** Form generated from reading UI file 'MainWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *fileQuitAction;
    QAction *helpAboutAction;
    QAction *fileOpenAction;
    QAction *viewZoomAllAction;
    QAction *viewZoomWindowAction;
    QAction *viewZoomOutAction;
    QAction *viewZoomInAction;
    QAction *createWayAction;
    QAction *editUndoAction;
    QAction *editRedoAction;
    QAction *editMoveAction;
    QAction *fileImportGDALAction;
    QAction *fileImportAction;
    QAction *fileDownloadAction;
    QAction *createLinearWayAction;
    QAction *editPropertiesAction;
    QAction *fileUploadAction;
    QAction *editRemoveAction;
    QAction *createRoadAction;
    QAction *createNodeAction;
    QAction *editReverseAction;
    QAction *viewGotoAction;
    QAction *createDoubleWayAction;
    QAction *createRoundaboutAction;
    QAction *fileNewAction;
    QAction *roadSplitAction;
    QAction *roadJoinAction;
    QAction *roadBreakAction;
    QAction *createRelationAction;
    QAction *createAreaAction;
    QAction *editMapStyleAction;
    QAction *mapStyleSaveAsAction;
    QAction *mapStyleLoadAction;
    QAction *createCurvedRoadAction;
    QAction *toolsPreferencesAction;
    QAction *exportOSMAllAction;
    QAction *exportOSMBinAllAction;
    QAction *editSelectAction;
    QAction *exportOSMViewportAction;
    QAction *exportOSMBinViewportAction;
    QAction *bookmarkAddAction;
    QAction *bookmarkRemoveAction;
    QAction *nodeMergeAction;
    QAction *fileSaveAsAction;
    QAction *fileSaveAction;
    QAction *fileDownloadMoreAction;
    QAction *action_Docks;
    QAction *windowPropertiesAction;
    QAction *windowLayersAction;
    QAction *windowInfoAction;
    QAction *nodeAlignAction;
    QAction *nodeSpreadAction;
    QAction *windowDirtyAction;
    QAction *viewDownloadedAction;
    QAction *editCopyAction;
    QAction *editPasteOverwriteAction;
    QAction *editPasteMergeAction;
    QAction *exportOSMSelectedAction;
    QAction *exportOSMBinSelectedAction;
    QAction *editPasteFeatureAction;
    QAction *exportOSMAction;
    QAction *exportOSMBinAction;
    QAction *featureCommitAction;
    QAction *exportGPXAction;
    QAction *exportKMLAction;
    QAction *windowToolbarAction;
    QAction *windowHideAllAction;
    QAction *windowShowAllAction;
    QAction *layersAddImageAction;
    QAction *renderNativeAction;
    QAction *viewTrackPointsAction;
    QAction *viewNamesAction;
    QAction *gpsConnectAction;
    QAction *gpsReplayAction;
    QAction *windowGPSAction;
    QAction *gpsDisconnectAction;
    QAction *gpsCenterAction;
    QAction *viewTrackSegmentsAction;
    QAction *viewScaleAction;
    QAction *viewRelationsAction;
    QAction *viewStyleForegroundAction;
    QAction *viewStyleBackgroundAction;
    QAction *viewStyleTouchupAction;
    QAction *gpsRecordAction;
    QAction *gpsPauseAction;
    QAction *windowGeoimageAction;
    QAction *toolsWorldOsbAction;
    QAction *toolsShortcutsAction;
    QAction *toolTemplatesLoadAction;
    QAction *toolTemplatesMergeAction;
    QAction *toolTemplatesSaveAction;
    QAction *relationAddMemberAction;
    QAction *relationRemoveMemberAction;
    QAction *viewArrowsNeverAction;
    QAction *viewArrowsOnewayAction;
    QAction *viewArrowsAlwaysAction;
    QAction *nodeDetachAction;
    QAction *fileWorkOfflineAction;
    QAction *renderSVGAction;
    QAction *windowStylesAction;
    QAction *toolsWMSServersAction;
    QAction *toolsTMSServersAction;
    QAction *toolsResetDiscardableAction;
    QAction *gpsPopupAction;
    QAction *cameraAction;
    QAction *roadCreateJunctionAction;
    QAction *editRotateAction;
    QAction *createPolygonAction;
    QAction *createRectangleAction;
    QAction *layersNewImageAction;
    QAction *windowFeatsAction;
    QAction *roadAddStreetNumbersAction;
    QAction *roadSubdivideAction;
    QAction *viewVirtualNodesAction;
    QAction *viewShowLatLonGridAction;
    QAction *viewLockZoomAction;
    QAction *toolsProjectionsAction;
    QAction *viewPhotosAction;
    QAction *exportOSCAction;
    QAction *featureDeleteAction;
    QAction *layersOpenstreetbugsAction;
    QAction *featureOsbClose;
    QAction *roadSimplifyAction;
    QAction *toolsFiltersAction;
    QAction *filterNoneAction;
    QAction *areaJoinAction;
    QAction *areaSplitAction;
    QAction *areaTerraceAction;
    QAction *toolsToolbarsAction;
    QAction *roadAxisAlignAction;
    QAction *filePrintAction;
    QAction *filePrintPreviewAction;
    QAction *filePropertiesAction;
    QAction *viewDirtyAction;
    QAction *layersNewDrawingAction;
    QAction *editCutAction;
    QAction *layersNewFilterAction;
    QAction *roadExtrudeAction;
    QAction *featureSelectAction;
    QAction *featureSelectChildrenAction;
    QAction *editScaleAction;
    QAction *fileSaveAsTemplateAction;
    QAction *actionCreate_Multipolygon;
    QAction *relationAddToMultipolygonAction;
    QAction *mapStyleSaveAction;
    QAction *exportGDALAction;
    QAction *roadBingExtractAction;
    QAction *toolsRebuildHistoryAction;
    QAction *layersMapdustAction;
    QAction *viewWireframeAction;
    QAction *featureSelectParentsAction;
    QAction *featureDownloadMissingChildrenAction;
    QAction *markBridgeAction;
    QWidget *centralWidget;
    QMenuBar *theMenuBar;
    QMenu *menuHelp;
    QMenu *menuCreate;
    QMenu *menuRoad;
    QMenu *menuEdit;
    QMenu *menuView;
    QMenu *menuBookmarks;
    QMenu *mnuProjections;
    QMenu *mnuAreaOpacity;
    QMenu *menuFile;
    QMenu *menuExport;
    QMenu *menuRecentOpen;
    QMenu *menuRecentImport;
    QMenu *menuTools;
    QMenu *menuStyles;
    QMenu *designerMenu;
    QMenu *menu_Node;
    QMenu *menuWindow;
    QMenu *menu_Docks;
    QMenu *menu_Feature;
    QMenu *menuLayers;
    QMenu *menuGps;
    QMenu *menuRelation;
    QMenu *menu_Show;
    QMenu *menuShow_directional_Arrows;
    QStatusBar *StatusBar;
    QToolBar *toolBar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(1100, 549);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/Icons/Merkaartor.xpm"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MainWindow->setWindowIcon(icon);
        fileQuitAction = new QAction(MainWindow);
        fileQuitAction->setObjectName("fileQuitAction");
#if QT_CONFIG(shortcut)
        fileQuitAction->setShortcut(QString::fromUtf8("Ctrl+Q"));
#endif // QT_CONFIG(shortcut)
        fileQuitAction->setMenuRole(QAction::QuitRole);
        helpAboutAction = new QAction(MainWindow);
        helpAboutAction->setObjectName("helpAboutAction");
#if QT_CONFIG(shortcut)
        helpAboutAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        helpAboutAction->setMenuRole(QAction::AboutRole);
        fileOpenAction = new QAction(MainWindow);
        fileOpenAction->setObjectName("fileOpenAction");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/Icons/actions/document_open.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileOpenAction->setIcon(icon1);
#if QT_CONFIG(shortcut)
        fileOpenAction->setShortcut(QString::fromUtf8("Ctrl+O"));
#endif // QT_CONFIG(shortcut)
        viewZoomAllAction = new QAction(MainWindow);
        viewZoomAllAction->setObjectName("viewZoomAllAction");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/Icons/actions/zoom_fit_best.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        viewZoomAllAction->setIcon(icon2);
#if QT_CONFIG(shortcut)
        viewZoomAllAction->setShortcut(QString::fromUtf8("F2"));
#endif // QT_CONFIG(shortcut)
        viewZoomWindowAction = new QAction(MainWindow);
        viewZoomWindowAction->setObjectName("viewZoomWindowAction");
#if QT_CONFIG(shortcut)
        viewZoomWindowAction->setShortcut(QString::fromUtf8("F3"));
#endif // QT_CONFIG(shortcut)
        viewZoomOutAction = new QAction(MainWindow);
        viewZoomOutAction->setObjectName("viewZoomOutAction");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/Icons/actions/zoom_out.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        viewZoomOutAction->setIcon(icon3);
#if QT_CONFIG(shortcut)
        viewZoomOutAction->setShortcut(QString::fromUtf8("-"));
#endif // QT_CONFIG(shortcut)
        viewZoomInAction = new QAction(MainWindow);
        viewZoomInAction->setObjectName("viewZoomInAction");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/Icons/actions/zoom_in.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        viewZoomInAction->setIcon(icon4);
#if QT_CONFIG(shortcut)
        viewZoomInAction->setShortcut(QString::fromUtf8("+"));
#endif // QT_CONFIG(shortcut)
        createWayAction = new QAction(MainWindow);
        createWayAction->setObjectName("createWayAction");
        editUndoAction = new QAction(MainWindow);
        editUndoAction->setObjectName("editUndoAction");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/Icons/actions/undo.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editUndoAction->setIcon(icon5);
#if QT_CONFIG(shortcut)
        editUndoAction->setShortcut(QString::fromUtf8("Ctrl+Z"));
#endif // QT_CONFIG(shortcut)
        editRedoAction = new QAction(MainWindow);
        editRedoAction->setObjectName("editRedoAction");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/Icons/actions/redo.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editRedoAction->setIcon(icon6);
#if QT_CONFIG(shortcut)
        editRedoAction->setShortcut(QString::fromUtf8("Ctrl+Y"));
#endif // QT_CONFIG(shortcut)
        editMoveAction = new QAction(MainWindow);
        editMoveAction->setObjectName("editMoveAction");
        editMoveAction->setCheckable(true);
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/Icons/actions/transform-move.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editMoveAction->setIcon(icon7);
#if QT_CONFIG(shortcut)
        editMoveAction->setShortcut(QString::fromUtf8("Ctrl+M"));
#endif // QT_CONFIG(shortcut)
        fileImportGDALAction = new QAction(MainWindow);
        fileImportGDALAction->setObjectName("fileImportGDALAction");
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/Icons/actions/document_import.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileImportGDALAction->setIcon(icon8);
#if QT_CONFIG(shortcut)
        fileImportGDALAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        fileImportAction = new QAction(MainWindow);
        fileImportAction->setObjectName("fileImportAction");
        fileImportAction->setIcon(icon8);
#if QT_CONFIG(shortcut)
        fileImportAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        fileDownloadAction = new QAction(MainWindow);
        fileDownloadAction->setObjectName("fileDownloadAction");
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/Icons/actions/download.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileDownloadAction->setIcon(icon9);
#if QT_CONFIG(shortcut)
        fileDownloadAction->setShortcut(QString::fromUtf8("Ctrl+D"));
#endif // QT_CONFIG(shortcut)
        createLinearWayAction = new QAction(MainWindow);
        createLinearWayAction->setObjectName("createLinearWayAction");
#if QT_CONFIG(shortcut)
        createLinearWayAction->setShortcut(QString::fromUtf8("L"));
#endif // QT_CONFIG(shortcut)
        editPropertiesAction = new QAction(MainWindow);
        editPropertiesAction->setObjectName("editPropertiesAction");
        editPropertiesAction->setCheckable(true);
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/Icons/actions/select.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editPropertiesAction->setIcon(icon10);
#if QT_CONFIG(shortcut)
        editPropertiesAction->setShortcut(QString::fromUtf8("Esc"));
#endif // QT_CONFIG(shortcut)
        fileUploadAction = new QAction(MainWindow);
        fileUploadAction->setObjectName("fileUploadAction");
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/Icons/actions/upload.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileUploadAction->setIcon(icon11);
#if QT_CONFIG(shortcut)
        fileUploadAction->setShortcut(QString::fromUtf8("Ctrl+U"));
#endif // QT_CONFIG(shortcut)
        editRemoveAction = new QAction(MainWindow);
        editRemoveAction->setObjectName("editRemoveAction");
        editRemoveAction->setEnabled(false);
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/Icons/actions/edit_delete.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editRemoveAction->setIcon(icon12);
#if QT_CONFIG(shortcut)
        editRemoveAction->setShortcut(QString::fromUtf8("Del"));
#endif // QT_CONFIG(shortcut)
        createRoadAction = new QAction(MainWindow);
        createRoadAction->setObjectName("createRoadAction");
        createRoadAction->setCheckable(true);
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/Icons/actions/create_road.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        createRoadAction->setIcon(icon13);
#if QT_CONFIG(shortcut)
        createRoadAction->setShortcut(QString::fromUtf8("Ctrl+R"));
#endif // QT_CONFIG(shortcut)
        createNodeAction = new QAction(MainWindow);
        createNodeAction->setObjectName("createNodeAction");
        createNodeAction->setCheckable(true);
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/Icons/actions/create_node.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        createNodeAction->setIcon(icon14);
#if QT_CONFIG(shortcut)
        createNodeAction->setShortcut(QString::fromUtf8("Ctrl+N"));
#endif // QT_CONFIG(shortcut)
        editReverseAction = new QAction(MainWindow);
        editReverseAction->setObjectName("editReverseAction");
        editReverseAction->setEnabled(false);
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/Icons/actions/reverse_road.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editReverseAction->setIcon(icon15);
#if QT_CONFIG(shortcut)
        editReverseAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewGotoAction = new QAction(MainWindow);
        viewGotoAction->setObjectName("viewGotoAction");
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/Icons/actions/goto.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        viewGotoAction->setIcon(icon16);
#if QT_CONFIG(shortcut)
        viewGotoAction->setShortcut(QString::fromUtf8("Ctrl+G"));
#endif // QT_CONFIG(shortcut)
        createDoubleWayAction = new QAction(MainWindow);
        createDoubleWayAction->setObjectName("createDoubleWayAction");
#if QT_CONFIG(shortcut)
        createDoubleWayAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        createRoundaboutAction = new QAction(MainWindow);
        createRoundaboutAction->setObjectName("createRoundaboutAction");
#if QT_CONFIG(shortcut)
        createRoundaboutAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        createRoundaboutAction->setMenuRole(QAction::NoRole);
        fileNewAction = new QAction(MainWindow);
        fileNewAction->setObjectName("fileNewAction");
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/Icons/actions/document_new.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileNewAction->setIcon(icon17);
#if QT_CONFIG(shortcut)
        fileNewAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadSplitAction = new QAction(MainWindow);
        roadSplitAction->setObjectName("roadSplitAction");
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/Icons/actions/split_road.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        roadSplitAction->setIcon(icon18);
#if QT_CONFIG(shortcut)
        roadSplitAction->setShortcut(QString::fromUtf8("Alt+S"));
#endif // QT_CONFIG(shortcut)
        roadJoinAction = new QAction(MainWindow);
        roadJoinAction->setObjectName("roadJoinAction");
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/Icons/actions/join_roads.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        roadJoinAction->setIcon(icon19);
#if QT_CONFIG(shortcut)
        roadJoinAction->setShortcut(QString::fromUtf8("Alt+J"));
#endif // QT_CONFIG(shortcut)
        roadBreakAction = new QAction(MainWindow);
        roadBreakAction->setObjectName("roadBreakAction");
        QIcon icon20;
        icon20.addFile(QString::fromUtf8(":/Icons/actions/break_apart_roads.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        roadBreakAction->setIcon(icon20);
#if QT_CONFIG(shortcut)
        roadBreakAction->setShortcut(QString::fromUtf8("Alt+B"));
#endif // QT_CONFIG(shortcut)
        createRelationAction = new QAction(MainWindow);
        createRelationAction->setObjectName("createRelationAction");
#if QT_CONFIG(shortcut)
        createRelationAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        createAreaAction = new QAction(MainWindow);
        createAreaAction->setObjectName("createAreaAction");
        createAreaAction->setCheckable(true);
        QIcon icon21;
        icon21.addFile(QString::fromUtf8(":/Icons/actions/create_area.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        createAreaAction->setIcon(icon21);
#if QT_CONFIG(shortcut)
        createAreaAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        editMapStyleAction = new QAction(MainWindow);
        editMapStyleAction->setObjectName("editMapStyleAction");
#if QT_CONFIG(shortcut)
        editMapStyleAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        mapStyleSaveAsAction = new QAction(MainWindow);
        mapStyleSaveAsAction->setObjectName("mapStyleSaveAsAction");
#if QT_CONFIG(shortcut)
        mapStyleSaveAsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        mapStyleLoadAction = new QAction(MainWindow);
        mapStyleLoadAction->setObjectName("mapStyleLoadAction");
#if QT_CONFIG(shortcut)
        mapStyleLoadAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        createCurvedRoadAction = new QAction(MainWindow);
        createCurvedRoadAction->setObjectName("createCurvedRoadAction");
        toolsPreferencesAction = new QAction(MainWindow);
        toolsPreferencesAction->setObjectName("toolsPreferencesAction");
        QIcon icon22;
        icon22.addFile(QString::fromUtf8(":/Icons/actions/preferences.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        toolsPreferencesAction->setIcon(icon22);
#if QT_CONFIG(shortcut)
        toolsPreferencesAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsPreferencesAction->setMenuRole(QAction::PreferencesRole);
        exportOSMAllAction = new QAction(MainWindow);
        exportOSMAllAction->setObjectName("exportOSMAllAction");
        exportOSMBinAllAction = new QAction(MainWindow);
        exportOSMBinAllAction->setObjectName("exportOSMBinAllAction");
        editSelectAction = new QAction(MainWindow);
        editSelectAction->setObjectName("editSelectAction");
        QIcon icon23;
        icon23.addFile(QString::fromUtf8(":/Icons/actions/find.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editSelectAction->setIcon(icon23);
#if QT_CONFIG(shortcut)
        editSelectAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportOSMViewportAction = new QAction(MainWindow);
        exportOSMViewportAction->setObjectName("exportOSMViewportAction");
        exportOSMBinViewportAction = new QAction(MainWindow);
        exportOSMBinViewportAction->setObjectName("exportOSMBinViewportAction");
        bookmarkAddAction = new QAction(MainWindow);
        bookmarkAddAction->setObjectName("bookmarkAddAction");
#if QT_CONFIG(shortcut)
        bookmarkAddAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        bookmarkRemoveAction = new QAction(MainWindow);
        bookmarkRemoveAction->setObjectName("bookmarkRemoveAction");
#if QT_CONFIG(shortcut)
        bookmarkRemoveAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        nodeMergeAction = new QAction(MainWindow);
        nodeMergeAction->setObjectName("nodeMergeAction");
#if QT_CONFIG(shortcut)
        nodeMergeAction->setShortcut(QString::fromUtf8("Alt+M"));
#endif // QT_CONFIG(shortcut)
        fileSaveAsAction = new QAction(MainWindow);
        fileSaveAsAction->setObjectName("fileSaveAsAction");
#if QT_CONFIG(shortcut)
        fileSaveAsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        fileSaveAction = new QAction(MainWindow);
        fileSaveAction->setObjectName("fileSaveAction");
        QIcon icon24;
        icon24.addFile(QString::fromUtf8(":/Icons/actions/save.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileSaveAction->setIcon(icon24);
#if QT_CONFIG(shortcut)
        fileSaveAction->setShortcut(QString::fromUtf8("Ctrl+S"));
#endif // QT_CONFIG(shortcut)
        fileDownloadMoreAction = new QAction(MainWindow);
        fileDownloadMoreAction->setObjectName("fileDownloadMoreAction");
        QIcon icon25;
        icon25.addFile(QString::fromUtf8(":/Icons/actions/download_more.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileDownloadMoreAction->setIcon(icon25);
#if QT_CONFIG(shortcut)
        fileDownloadMoreAction->setShortcut(QString::fromUtf8("Ctrl+Shift+D"));
#endif // QT_CONFIG(shortcut)
        action_Docks = new QAction(MainWindow);
        action_Docks->setObjectName("action_Docks");
        windowPropertiesAction = new QAction(MainWindow);
        windowPropertiesAction->setObjectName("windowPropertiesAction");
        windowPropertiesAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowPropertiesAction->setShortcut(QString::fromUtf8("Ctrl+P"));
#endif // QT_CONFIG(shortcut)
        windowLayersAction = new QAction(MainWindow);
        windowLayersAction->setObjectName("windowLayersAction");
        windowLayersAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowLayersAction->setShortcut(QString::fromUtf8("Ctrl+L"));
#endif // QT_CONFIG(shortcut)
        windowInfoAction = new QAction(MainWindow);
        windowInfoAction->setObjectName("windowInfoAction");
        windowInfoAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowInfoAction->setShortcut(QString::fromUtf8("Ctrl+I"));
#endif // QT_CONFIG(shortcut)
        nodeAlignAction = new QAction(MainWindow);
        nodeAlignAction->setObjectName("nodeAlignAction");
        QIcon icon26;
        icon26.addFile(QString::fromUtf8(":/Icons/actions/align_nodes.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nodeAlignAction->setIcon(icon26);
#if QT_CONFIG(shortcut)
        nodeAlignAction->setShortcut(QString::fromUtf8("Alt+A"));
#endif // QT_CONFIG(shortcut)
        nodeSpreadAction = new QAction(MainWindow);
        nodeSpreadAction->setObjectName("nodeSpreadAction");
        QIcon icon27;
        icon27.addFile(QString::fromUtf8(":/Icons/actions/spread_nodes.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nodeSpreadAction->setIcon(icon27);
#if QT_CONFIG(shortcut)
        nodeSpreadAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowDirtyAction = new QAction(MainWindow);
        windowDirtyAction->setObjectName("windowDirtyAction");
        windowDirtyAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowDirtyAction->setShortcut(QString::fromUtf8("Ctrl+T"));
#endif // QT_CONFIG(shortcut)
        viewDownloadedAction = new QAction(MainWindow);
        viewDownloadedAction->setObjectName("viewDownloadedAction");
        viewDownloadedAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewDownloadedAction->setShortcut(QString::fromUtf8("Ctrl+Alt+A"));
#endif // QT_CONFIG(shortcut)
        editCopyAction = new QAction(MainWindow);
        editCopyAction->setObjectName("editCopyAction");
        QIcon icon28;
        icon28.addFile(QString::fromUtf8(":/Icons/actions/edit_copy.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editCopyAction->setIcon(icon28);
#if QT_CONFIG(shortcut)
        editCopyAction->setShortcut(QString::fromUtf8("Ctrl+C"));
#endif // QT_CONFIG(shortcut)
        editPasteOverwriteAction = new QAction(MainWindow);
        editPasteOverwriteAction->setObjectName("editPasteOverwriteAction");
#if QT_CONFIG(shortcut)
        editPasteOverwriteAction->setShortcut(QString::fromUtf8("Ctrl+V, O"));
#endif // QT_CONFIG(shortcut)
        editPasteMergeAction = new QAction(MainWindow);
        editPasteMergeAction->setObjectName("editPasteMergeAction");
        QIcon icon29;
        icon29.addFile(QString::fromUtf8(":/Icons/actions/edit_paste_tags.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editPasteMergeAction->setIcon(icon29);
#if QT_CONFIG(shortcut)
        editPasteMergeAction->setShortcut(QString::fromUtf8("Ctrl+V, M"));
#endif // QT_CONFIG(shortcut)
        exportOSMSelectedAction = new QAction(MainWindow);
        exportOSMSelectedAction->setObjectName("exportOSMSelectedAction");
        exportOSMBinSelectedAction = new QAction(MainWindow);
        exportOSMBinSelectedAction->setObjectName("exportOSMBinSelectedAction");
        editPasteFeatureAction = new QAction(MainWindow);
        editPasteFeatureAction->setObjectName("editPasteFeatureAction");
        QIcon icon30;
        icon30.addFile(QString::fromUtf8(":/Icons/actions/edit_paste.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editPasteFeatureAction->setIcon(icon30);
#if QT_CONFIG(shortcut)
        editPasteFeatureAction->setShortcut(QString::fromUtf8("Ctrl+V, F"));
#endif // QT_CONFIG(shortcut)
        exportOSMAction = new QAction(MainWindow);
        exportOSMAction->setObjectName("exportOSMAction");
#if QT_CONFIG(shortcut)
        exportOSMAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportOSMBinAction = new QAction(MainWindow);
        exportOSMBinAction->setObjectName("exportOSMBinAction");
        featureCommitAction = new QAction(MainWindow);
        featureCommitAction->setObjectName("featureCommitAction");
#if QT_CONFIG(shortcut)
        featureCommitAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportGPXAction = new QAction(MainWindow);
        exportGPXAction->setObjectName("exportGPXAction");
#if QT_CONFIG(shortcut)
        exportGPXAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportKMLAction = new QAction(MainWindow);
        exportKMLAction->setObjectName("exportKMLAction");
#if QT_CONFIG(shortcut)
        exportKMLAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowToolbarAction = new QAction(MainWindow);
        windowToolbarAction->setObjectName("windowToolbarAction");
#if QT_CONFIG(shortcut)
        windowToolbarAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowHideAllAction = new QAction(MainWindow);
        windowHideAllAction->setObjectName("windowHideAllAction");
#if QT_CONFIG(shortcut)
        windowHideAllAction->setShortcut(QString::fromUtf8("Ctrl+F"));
#endif // QT_CONFIG(shortcut)
        windowShowAllAction = new QAction(MainWindow);
        windowShowAllAction->setObjectName("windowShowAllAction");
#if QT_CONFIG(shortcut)
        windowShowAllAction->setShortcut(QString::fromUtf8("Ctrl+F"));
#endif // QT_CONFIG(shortcut)
        windowShowAllAction->setVisible(false);
        layersAddImageAction = new QAction(MainWindow);
        layersAddImageAction->setObjectName("layersAddImageAction");
        renderNativeAction = new QAction(MainWindow);
        renderNativeAction->setObjectName("renderNativeAction");
        viewTrackPointsAction = new QAction(MainWindow);
        viewTrackPointsAction->setObjectName("viewTrackPointsAction");
        viewTrackPointsAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewTrackPointsAction->setShortcut(QString::fromUtf8("Ctrl+Alt+P"));
#endif // QT_CONFIG(shortcut)
        viewNamesAction = new QAction(MainWindow);
        viewNamesAction->setObjectName("viewNamesAction");
        viewNamesAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewNamesAction->setShortcut(QString::fromUtf8("Ctrl+Alt+N"));
#endif // QT_CONFIG(shortcut)
        gpsConnectAction = new QAction(MainWindow);
        gpsConnectAction->setObjectName("gpsConnectAction");
#if QT_CONFIG(shortcut)
        gpsConnectAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        gpsReplayAction = new QAction(MainWindow);
        gpsReplayAction->setObjectName("gpsReplayAction");
#if QT_CONFIG(shortcut)
        gpsReplayAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowGPSAction = new QAction(MainWindow);
        windowGPSAction->setObjectName("windowGPSAction");
        windowGPSAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowGPSAction->setShortcut(QString::fromUtf8("Ctrl+W"));
#endif // QT_CONFIG(shortcut)
        gpsDisconnectAction = new QAction(MainWindow);
        gpsDisconnectAction->setObjectName("gpsDisconnectAction");
        gpsDisconnectAction->setEnabled(false);
#if QT_CONFIG(shortcut)
        gpsDisconnectAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        gpsCenterAction = new QAction(MainWindow);
        gpsCenterAction->setObjectName("gpsCenterAction");
        gpsCenterAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        gpsCenterAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewTrackSegmentsAction = new QAction(MainWindow);
        viewTrackSegmentsAction->setObjectName("viewTrackSegmentsAction");
        viewTrackSegmentsAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewTrackSegmentsAction->setShortcut(QString::fromUtf8("Ctrl+Alt+T"));
#endif // QT_CONFIG(shortcut)
        viewScaleAction = new QAction(MainWindow);
        viewScaleAction->setObjectName("viewScaleAction");
        viewScaleAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewScaleAction->setShortcut(QString::fromUtf8("Ctrl+Alt+S"));
#endif // QT_CONFIG(shortcut)
        viewRelationsAction = new QAction(MainWindow);
        viewRelationsAction->setObjectName("viewRelationsAction");
        viewRelationsAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewRelationsAction->setShortcut(QString::fromUtf8("Ctrl+Alt+R"));
#endif // QT_CONFIG(shortcut)
        viewStyleForegroundAction = new QAction(MainWindow);
        viewStyleForegroundAction->setObjectName("viewStyleForegroundAction");
        viewStyleForegroundAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewStyleForegroundAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewStyleBackgroundAction = new QAction(MainWindow);
        viewStyleBackgroundAction->setObjectName("viewStyleBackgroundAction");
        viewStyleBackgroundAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewStyleBackgroundAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewStyleTouchupAction = new QAction(MainWindow);
        viewStyleTouchupAction->setObjectName("viewStyleTouchupAction");
        viewStyleTouchupAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewStyleTouchupAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        gpsRecordAction = new QAction(MainWindow);
        gpsRecordAction->setObjectName("gpsRecordAction");
        gpsRecordAction->setCheckable(true);
        gpsRecordAction->setEnabled(false);
#if QT_CONFIG(shortcut)
        gpsRecordAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        gpsPauseAction = new QAction(MainWindow);
        gpsPauseAction->setObjectName("gpsPauseAction");
        gpsPauseAction->setCheckable(true);
        gpsPauseAction->setEnabled(false);
#if QT_CONFIG(shortcut)
        gpsPauseAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowGeoimageAction = new QAction(MainWindow);
        windowGeoimageAction->setObjectName("windowGeoimageAction");
        windowGeoimageAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowGeoimageAction->setShortcut(QString::fromUtf8("Ctrl+E"));
#endif // QT_CONFIG(shortcut)
        toolsWorldOsbAction = new QAction(MainWindow);
        toolsWorldOsbAction->setObjectName("toolsWorldOsbAction");
#if QT_CONFIG(shortcut)
        toolsWorldOsbAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsWorldOsbAction->setVisible(false);
        toolsShortcutsAction = new QAction(MainWindow);
        toolsShortcutsAction->setObjectName("toolsShortcutsAction");
#if QT_CONFIG(shortcut)
        toolsShortcutsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolTemplatesLoadAction = new QAction(MainWindow);
        toolTemplatesLoadAction->setObjectName("toolTemplatesLoadAction");
#if QT_CONFIG(shortcut)
        toolTemplatesLoadAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolTemplatesMergeAction = new QAction(MainWindow);
        toolTemplatesMergeAction->setObjectName("toolTemplatesMergeAction");
#if QT_CONFIG(shortcut)
        toolTemplatesMergeAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolTemplatesSaveAction = new QAction(MainWindow);
        toolTemplatesSaveAction->setObjectName("toolTemplatesSaveAction");
#if QT_CONFIG(shortcut)
        toolTemplatesSaveAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        relationAddMemberAction = new QAction(MainWindow);
        relationAddMemberAction->setObjectName("relationAddMemberAction");
#if QT_CONFIG(shortcut)
        relationAddMemberAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        relationRemoveMemberAction = new QAction(MainWindow);
        relationRemoveMemberAction->setObjectName("relationRemoveMemberAction");
#if QT_CONFIG(shortcut)
        relationRemoveMemberAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewArrowsNeverAction = new QAction(MainWindow);
        viewArrowsNeverAction->setObjectName("viewArrowsNeverAction");
        viewArrowsNeverAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewArrowsNeverAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewArrowsOnewayAction = new QAction(MainWindow);
        viewArrowsOnewayAction->setObjectName("viewArrowsOnewayAction");
        viewArrowsOnewayAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewArrowsOnewayAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewArrowsAlwaysAction = new QAction(MainWindow);
        viewArrowsAlwaysAction->setObjectName("viewArrowsAlwaysAction");
        viewArrowsAlwaysAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewArrowsAlwaysAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        nodeDetachAction = new QAction(MainWindow);
        nodeDetachAction->setObjectName("nodeDetachAction");
        QIcon icon31;
        icon31.addFile(QString::fromUtf8(":/Icons/actions/detach_node.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nodeDetachAction->setIcon(icon31);
#if QT_CONFIG(shortcut)
        nodeDetachAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        fileWorkOfflineAction = new QAction(MainWindow);
        fileWorkOfflineAction->setObjectName("fileWorkOfflineAction");
        fileWorkOfflineAction->setCheckable(true);
        QIcon icon32;
        icon32.addFile(QString::fromUtf8(":/Icons/actions/offline.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        fileWorkOfflineAction->setIcon(icon32);
#if QT_CONFIG(shortcut)
        fileWorkOfflineAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        renderSVGAction = new QAction(MainWindow);
        renderSVGAction->setObjectName("renderSVGAction");
        windowStylesAction = new QAction(MainWindow);
        windowStylesAction->setObjectName("windowStylesAction");
        windowStylesAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowStylesAction->setShortcut(QString::fromUtf8("Ctrl+B"));
#endif // QT_CONFIG(shortcut)
        toolsWMSServersAction = new QAction(MainWindow);
        toolsWMSServersAction->setObjectName("toolsWMSServersAction");
#if QT_CONFIG(shortcut)
        toolsWMSServersAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsTMSServersAction = new QAction(MainWindow);
        toolsTMSServersAction->setObjectName("toolsTMSServersAction");
#if QT_CONFIG(shortcut)
        toolsTMSServersAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsResetDiscardableAction = new QAction(MainWindow);
        toolsResetDiscardableAction->setObjectName("toolsResetDiscardableAction");
#if QT_CONFIG(shortcut)
        toolsResetDiscardableAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        gpsPopupAction = new QAction(MainWindow);
        gpsPopupAction->setObjectName("gpsPopupAction");
        QIcon icon33;
        icon33.addFile(QString::fromUtf8(":/Icons/actions/GPS.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        gpsPopupAction->setIcon(icon33);
        cameraAction = new QAction(MainWindow);
        cameraAction->setObjectName("cameraAction");
        QIcon icon34;
        icon34.addFile(QString::fromUtf8(":/Icons/actions/camera.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        cameraAction->setIcon(icon34);
        roadCreateJunctionAction = new QAction(MainWindow);
        roadCreateJunctionAction->setObjectName("roadCreateJunctionAction");
#if QT_CONFIG(shortcut)
        roadCreateJunctionAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        editRotateAction = new QAction(MainWindow);
        editRotateAction->setObjectName("editRotateAction");
        editRotateAction->setCheckable(true);
        QIcon icon35;
        icon35.addFile(QString::fromUtf8(":/Icons/actions/transform-rotate.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editRotateAction->setIcon(icon35);
#if QT_CONFIG(shortcut)
        editRotateAction->setShortcut(QString::fromUtf8("Ctrl+A"));
#endif // QT_CONFIG(shortcut)
        createPolygonAction = new QAction(MainWindow);
        createPolygonAction->setObjectName("createPolygonAction");
#if QT_CONFIG(shortcut)
        createPolygonAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        createRectangleAction = new QAction(MainWindow);
        createRectangleAction->setObjectName("createRectangleAction");
#if QT_CONFIG(shortcut)
        createRectangleAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        layersNewImageAction = new QAction(MainWindow);
        layersNewImageAction->setObjectName("layersNewImageAction");
#if QT_CONFIG(shortcut)
        layersNewImageAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        windowFeatsAction = new QAction(MainWindow);
        windowFeatsAction->setObjectName("windowFeatsAction");
        windowFeatsAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        windowFeatsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadAddStreetNumbersAction = new QAction(MainWindow);
        roadAddStreetNumbersAction->setObjectName("roadAddStreetNumbersAction");
#if QT_CONFIG(shortcut)
        roadAddStreetNumbersAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadSubdivideAction = new QAction(MainWindow);
        roadSubdivideAction->setObjectName("roadSubdivideAction");
        QIcon icon36;
        icon36.addFile(QString::fromUtf8(":/Icons/actions/subdivide_road.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        roadSubdivideAction->setIcon(icon36);
#if QT_CONFIG(shortcut)
        roadSubdivideAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewVirtualNodesAction = new QAction(MainWindow);
        viewVirtualNodesAction->setObjectName("viewVirtualNodesAction");
        viewVirtualNodesAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewVirtualNodesAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewShowLatLonGridAction = new QAction(MainWindow);
        viewShowLatLonGridAction->setObjectName("viewShowLatLonGridAction");
        viewShowLatLonGridAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewShowLatLonGridAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewLockZoomAction = new QAction(MainWindow);
        viewLockZoomAction->setObjectName("viewLockZoomAction");
        viewLockZoomAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewLockZoomAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsProjectionsAction = new QAction(MainWindow);
        toolsProjectionsAction->setObjectName("toolsProjectionsAction");
#if QT_CONFIG(shortcut)
        toolsProjectionsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewPhotosAction = new QAction(MainWindow);
        viewPhotosAction->setObjectName("viewPhotosAction");
        viewPhotosAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewPhotosAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportOSCAction = new QAction(MainWindow);
        exportOSCAction->setObjectName("exportOSCAction");
#if QT_CONFIG(shortcut)
        exportOSCAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        featureDeleteAction = new QAction(MainWindow);
        featureDeleteAction->setObjectName("featureDeleteAction");
#if QT_CONFIG(shortcut)
        featureDeleteAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        layersOpenstreetbugsAction = new QAction(MainWindow);
        layersOpenstreetbugsAction->setObjectName("layersOpenstreetbugsAction");
        featureOsbClose = new QAction(MainWindow);
        featureOsbClose->setObjectName("featureOsbClose");
        roadSimplifyAction = new QAction(MainWindow);
        roadSimplifyAction->setObjectName("roadSimplifyAction");
#if QT_CONFIG(shortcut)
        roadSimplifyAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsFiltersAction = new QAction(MainWindow);
        toolsFiltersAction->setObjectName("toolsFiltersAction");
#if QT_CONFIG(shortcut)
        toolsFiltersAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        filterNoneAction = new QAction(MainWindow);
        filterNoneAction->setObjectName("filterNoneAction");
        filterNoneAction->setCheckable(true);
        areaJoinAction = new QAction(MainWindow);
        areaJoinAction->setObjectName("areaJoinAction");
        QIcon icon37;
        icon37.addFile(QString::fromUtf8(":/Icons/actions/join_areas.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        areaJoinAction->setIcon(icon37);
#if QT_CONFIG(shortcut)
        areaJoinAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        areaSplitAction = new QAction(MainWindow);
        areaSplitAction->setObjectName("areaSplitAction");
        QIcon icon38;
        icon38.addFile(QString::fromUtf8(":/Icons/actions/split_area.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        areaSplitAction->setIcon(icon38);
#if QT_CONFIG(shortcut)
        areaSplitAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        areaTerraceAction = new QAction(MainWindow);
        areaTerraceAction->setObjectName("areaTerraceAction");
        QIcon icon39;
        icon39.addFile(QString::fromUtf8(":/Icons/actions/terrace_building.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        areaTerraceAction->setIcon(icon39);
#if QT_CONFIG(shortcut)
        areaTerraceAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsToolbarsAction = new QAction(MainWindow);
        toolsToolbarsAction->setObjectName("toolsToolbarsAction");
#if QT_CONFIG(shortcut)
        toolsToolbarsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadAxisAlignAction = new QAction(MainWindow);
        roadAxisAlignAction->setObjectName("roadAxisAlignAction");
        QIcon icon40;
        icon40.addFile(QString::fromUtf8(":/Icons/actions/axisalign.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        roadAxisAlignAction->setIcon(icon40);
#if QT_CONFIG(shortcut)
        roadAxisAlignAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        filePrintAction = new QAction(MainWindow);
        filePrintAction->setObjectName("filePrintAction");
#if QT_CONFIG(shortcut)
        filePrintAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        filePrintPreviewAction = new QAction(MainWindow);
        filePrintPreviewAction->setObjectName("filePrintPreviewAction");
        filePropertiesAction = new QAction(MainWindow);
        filePropertiesAction->setObjectName("filePropertiesAction");
#if QT_CONFIG(shortcut)
        filePropertiesAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewDirtyAction = new QAction(MainWindow);
        viewDirtyAction->setObjectName("viewDirtyAction");
        viewDirtyAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewDirtyAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        layersNewDrawingAction = new QAction(MainWindow);
        layersNewDrawingAction->setObjectName("layersNewDrawingAction");
#if QT_CONFIG(shortcut)
        layersNewDrawingAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        editCutAction = new QAction(MainWindow);
        editCutAction->setObjectName("editCutAction");
        QIcon icon41;
        icon41.addFile(QString::fromUtf8(":/Icons/actions/edit-cut.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editCutAction->setIcon(icon41);
#if QT_CONFIG(shortcut)
        editCutAction->setShortcut(QString::fromUtf8("Ctrl+X"));
#endif // QT_CONFIG(shortcut)
        layersNewFilterAction = new QAction(MainWindow);
        layersNewFilterAction->setObjectName("layersNewFilterAction");
#if QT_CONFIG(shortcut)
        layersNewFilterAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadExtrudeAction = new QAction(MainWindow);
        roadExtrudeAction->setObjectName("roadExtrudeAction");
        roadExtrudeAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        roadExtrudeAction->setShortcut(QString::fromUtf8("Alt+X"));
#endif // QT_CONFIG(shortcut)
        featureSelectAction = new QAction(MainWindow);
        featureSelectAction->setObjectName("featureSelectAction");
        featureSelectChildrenAction = new QAction(MainWindow);
        featureSelectChildrenAction->setObjectName("featureSelectChildrenAction");
#if QT_CONFIG(shortcut)
        featureSelectChildrenAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        editScaleAction = new QAction(MainWindow);
        editScaleAction->setObjectName("editScaleAction");
        editScaleAction->setCheckable(true);
        QIcon icon42;
        icon42.addFile(QString::fromUtf8(":/Icons/actions/transform-scale.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        editScaleAction->setIcon(icon42);
#if QT_CONFIG(shortcut)
        editScaleAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        fileSaveAsTemplateAction = new QAction(MainWindow);
        fileSaveAsTemplateAction->setObjectName("fileSaveAsTemplateAction");
#if QT_CONFIG(shortcut)
        fileSaveAsTemplateAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionCreate_Multipolygon = new QAction(MainWindow);
        actionCreate_Multipolygon->setObjectName("actionCreate_Multipolygon");
        relationAddToMultipolygonAction = new QAction(MainWindow);
        relationAddToMultipolygonAction->setObjectName("relationAddToMultipolygonAction");
#if QT_CONFIG(shortcut)
        relationAddToMultipolygonAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        mapStyleSaveAction = new QAction(MainWindow);
        mapStyleSaveAction->setObjectName("mapStyleSaveAction");
#if QT_CONFIG(shortcut)
        mapStyleSaveAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        exportGDALAction = new QAction(MainWindow);
        exportGDALAction->setObjectName("exportGDALAction");
#if QT_CONFIG(shortcut)
        exportGDALAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        roadBingExtractAction = new QAction(MainWindow);
        roadBingExtractAction->setObjectName("roadBingExtractAction");
#if QT_CONFIG(shortcut)
        roadBingExtractAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        toolsRebuildHistoryAction = new QAction(MainWindow);
        toolsRebuildHistoryAction->setObjectName("toolsRebuildHistoryAction");
#if QT_CONFIG(shortcut)
        toolsRebuildHistoryAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        layersMapdustAction = new QAction(MainWindow);
        layersMapdustAction->setObjectName("layersMapdustAction");
#if QT_CONFIG(shortcut)
        layersMapdustAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        viewWireframeAction = new QAction(MainWindow);
        viewWireframeAction->setObjectName("viewWireframeAction");
        viewWireframeAction->setCheckable(true);
#if QT_CONFIG(shortcut)
        viewWireframeAction->setShortcut(QString::fromUtf8("Ctrl+Alt+W"));
#endif // QT_CONFIG(shortcut)
        featureSelectParentsAction = new QAction(MainWindow);
        featureSelectParentsAction->setObjectName("featureSelectParentsAction");
#if QT_CONFIG(shortcut)
        featureSelectParentsAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        featureDownloadMissingChildrenAction = new QAction(MainWindow);
        featureDownloadMissingChildrenAction->setObjectName("featureDownloadMissingChildrenAction");
#if QT_CONFIG(shortcut)
        featureDownloadMissingChildrenAction->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        markBridgeAction = new QAction(MainWindow);
        markBridgeAction->setObjectName("markBridgeAction");
        markBridgeAction->setCheckable(true);
        QIcon icon43;
        icon43.addFile(QString::fromUtf8(":/Icons/actions/build_bridge.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        markBridgeAction->setIcon(icon43);
#if QT_CONFIG(shortcut)
        markBridgeAction->setShortcut(QString::fromUtf8("Ctrl+B"));
#endif // QT_CONFIG(shortcut)
        centralWidget = new QWidget(MainWindow);
        centralWidget->setObjectName("centralWidget");
        MainWindow->setCentralWidget(centralWidget);
        theMenuBar = new QMenuBar(MainWindow);
        theMenuBar->setObjectName("theMenuBar");
        theMenuBar->setGeometry(QRect(0, 0, 1100, 19));
        menuHelp = new QMenu(theMenuBar);
        menuHelp->setObjectName("menuHelp");
        menuCreate = new QMenu(theMenuBar);
        menuCreate->setObjectName("menuCreate");
        menuRoad = new QMenu(theMenuBar);
        menuRoad->setObjectName("menuRoad");
        menuEdit = new QMenu(theMenuBar);
        menuEdit->setObjectName("menuEdit");
        menuView = new QMenu(theMenuBar);
        menuView->setObjectName("menuView");
        menuBookmarks = new QMenu(menuView);
        menuBookmarks->setObjectName("menuBookmarks");
        mnuProjections = new QMenu(menuView);
        mnuProjections->setObjectName("mnuProjections");
        mnuAreaOpacity = new QMenu(menuView);
        mnuAreaOpacity->setObjectName("mnuAreaOpacity");
        menuFile = new QMenu(theMenuBar);
        menuFile->setObjectName("menuFile");
        menuExport = new QMenu(menuFile);
        menuExport->setObjectName("menuExport");
        menuRecentOpen = new QMenu(menuFile);
        menuRecentOpen->setObjectName("menuRecentOpen");
        menuRecentImport = new QMenu(menuFile);
        menuRecentImport->setObjectName("menuRecentImport");
        menuTools = new QMenu(theMenuBar);
        menuTools->setObjectName("menuTools");
        menuStyles = new QMenu(menuTools);
        menuStyles->setObjectName("menuStyles");
        designerMenu = new QMenu(menuTools);
        designerMenu->setObjectName("designerMenu");
        menu_Node = new QMenu(theMenuBar);
        menu_Node->setObjectName("menu_Node");
        menuWindow = new QMenu(theMenuBar);
        menuWindow->setObjectName("menuWindow");
        menu_Docks = new QMenu(menuWindow);
        menu_Docks->setObjectName("menu_Docks");
        menu_Feature = new QMenu(theMenuBar);
        menu_Feature->setObjectName("menu_Feature");
        menuLayers = new QMenu(theMenuBar);
        menuLayers->setObjectName("menuLayers");
        menuGps = new QMenu(theMenuBar);
        menuGps->setObjectName("menuGps");
        menuRelation = new QMenu(theMenuBar);
        menuRelation->setObjectName("menuRelation");
        menu_Show = new QMenu(theMenuBar);
        menu_Show->setObjectName("menu_Show");
        menuShow_directional_Arrows = new QMenu(menu_Show);
        menuShow_directional_Arrows->setObjectName("menuShow_directional_Arrows");
        MainWindow->setMenuBar(theMenuBar);
        StatusBar = new QStatusBar(MainWindow);
        StatusBar->setObjectName("StatusBar");
        MainWindow->setStatusBar(StatusBar);
        toolBar = new QToolBar(MainWindow);
        toolBar->setObjectName("toolBar");
        toolBar->setEnabled(true);
        toolBar->setOrientation(Qt::Horizontal);
        MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, toolBar);

        theMenuBar->addAction(menuFile->menuAction());
        theMenuBar->addAction(menuEdit->menuAction());
        theMenuBar->addAction(menuView->menuAction());
        theMenuBar->addAction(menu_Show->menuAction());
        theMenuBar->addAction(menuGps->menuAction());
        theMenuBar->addAction(menuLayers->menuAction());
        theMenuBar->addAction(menuCreate->menuAction());
        theMenuBar->addAction(menu_Feature->menuAction());
        theMenuBar->addAction(menu_Node->menuAction());
        theMenuBar->addAction(menuRoad->menuAction());
        theMenuBar->addAction(menuRelation->menuAction());
        theMenuBar->addAction(menuTools->menuAction());
        theMenuBar->addAction(menuWindow->menuAction());
        theMenuBar->addSeparator();
        theMenuBar->addAction(menuHelp->menuAction());
        menuHelp->addAction(helpAboutAction);
        menuCreate->addAction(createNodeAction);
        menuCreate->addAction(createRoadAction);
        menuCreate->addAction(createDoubleWayAction);
        menuCreate->addAction(createRoundaboutAction);
        menuCreate->addAction(createRectangleAction);
        menuCreate->addAction(createPolygonAction);
        menuCreate->addAction(createAreaAction);
        menuCreate->addAction(createRelationAction);
        menuRoad->addAction(roadSplitAction);
        menuRoad->addAction(roadBreakAction);
        menuRoad->addAction(roadJoinAction);
        menuRoad->addAction(editReverseAction);
        menuRoad->addAction(roadSimplifyAction);
        menuRoad->addAction(roadCreateJunctionAction);
        menuRoad->addAction(roadAddStreetNumbersAction);
        menuRoad->addAction(roadSubdivideAction);
        menuRoad->addAction(areaJoinAction);
        menuRoad->addAction(areaSplitAction);
        menuRoad->addAction(areaTerraceAction);
        menuRoad->addAction(roadAxisAlignAction);
        menuRoad->addSeparator();
        menuRoad->addAction(roadBingExtractAction);
        menuEdit->addAction(editUndoAction);
        menuEdit->addAction(editRedoAction);
        menuEdit->addSeparator();
        menuEdit->addAction(editCutAction);
        menuEdit->addAction(editCopyAction);
        menuEdit->addAction(editPasteFeatureAction);
        menuEdit->addAction(editPasteMergeAction);
        menuEdit->addAction(editPasteOverwriteAction);
        menuEdit->addSeparator();
        menuEdit->addAction(editRemoveAction);
        menuEdit->addAction(editMoveAction);
        menuEdit->addAction(editRotateAction);
        menuEdit->addAction(editScaleAction);
        menuEdit->addAction(roadExtrudeAction);
        menuEdit->addAction(editPropertiesAction);
        menuEdit->addSeparator();
        menuEdit->addAction(editSelectAction);
        menuView->addAction(viewZoomAllAction);
        menuView->addAction(viewZoomWindowAction);
        menuView->addAction(viewZoomOutAction);
        menuView->addAction(viewZoomInAction);
        menuView->addAction(viewLockZoomAction);
        menuView->addSeparator();
        menuView->addAction(viewWireframeAction);
        menuView->addAction(mnuAreaOpacity->menuAction());
        menuView->addAction(mnuProjections->menuAction());
        menuView->addSeparator();
        menuView->addAction(viewGotoAction);
        menuView->addAction(menuBookmarks->menuAction());
        menuBookmarks->addAction(bookmarkAddAction);
        menuBookmarks->addAction(bookmarkRemoveAction);
        menuBookmarks->addSeparator();
        menuFile->addAction(fileNewAction);
        menuFile->addAction(fileOpenAction);
        menuFile->addAction(fileImportAction);
        menuFile->addAction(fileImportGDALAction);
        menuFile->addAction(menuRecentOpen->menuAction());
        menuFile->addAction(menuRecentImport->menuAction());
        menuFile->addSeparator();
        menuFile->addAction(menuExport->menuAction());
        menuFile->addSeparator();
        menuFile->addAction(fileSaveAction);
        menuFile->addAction(fileSaveAsAction);
        menuFile->addAction(fileSaveAsTemplateAction);
        menuFile->addSeparator();
        menuFile->addAction(fileDownloadAction);
        menuFile->addAction(fileDownloadMoreAction);
        menuFile->addAction(fileUploadAction);
        menuFile->addSeparator();
        menuFile->addAction(filePrintAction);
        menuFile->addSeparator();
        menuFile->addAction(filePropertiesAction);
        menuFile->addSeparator();
        menuFile->addAction(fileWorkOfflineAction);
        menuFile->addAction(fileQuitAction);
        menuExport->addAction(exportOSMAction);
        menuExport->addAction(exportOSCAction);
        menuExport->addAction(exportGPXAction);
        menuExport->addAction(exportKMLAction);
        menuExport->addAction(exportGDALAction);
        menuTools->addAction(menuStyles->menuAction());
        menuTools->addAction(designerMenu->menuAction());
        menuTools->addSeparator();
        menuTools->addAction(toolsToolbarsAction);
        menuTools->addAction(toolsShortcutsAction);
        menuTools->addSeparator();
        menuTools->addAction(toolsWorldOsbAction);
        menuTools->addAction(toolsTMSServersAction);
        menuTools->addAction(toolsWMSServersAction);
        menuTools->addAction(toolsProjectionsAction);
        menuTools->addAction(toolsFiltersAction);
        menuTools->addSeparator();
        menuTools->addAction(toolsResetDiscardableAction);
        menuTools->addAction(toolsRebuildHistoryAction);
        menuTools->addSeparator();
        menuTools->addAction(toolsPreferencesAction);
        menuStyles->addAction(editMapStyleAction);
        menuStyles->addSeparator();
        menuStyles->addAction(mapStyleSaveAction);
        menuStyles->addAction(mapStyleSaveAsAction);
        menuStyles->addAction(mapStyleLoadAction);
        menuStyles->addSeparator();
        designerMenu->addAction(toolTemplatesSaveAction);
        designerMenu->addAction(toolTemplatesMergeAction);
        designerMenu->addAction(toolTemplatesLoadAction);
        menu_Node->addAction(nodeMergeAction);
        menu_Node->addAction(nodeAlignAction);
        menu_Node->addAction(nodeSpreadAction);
        menu_Node->addAction(nodeDetachAction);
        menuWindow->addAction(menu_Docks->menuAction());
        menuWindow->addAction(windowToolbarAction);
        menuWindow->addSeparator();
        menuWindow->addAction(windowHideAllAction);
        menuWindow->addAction(windowShowAllAction);
        menu_Docks->addAction(windowPropertiesAction);
        menu_Docks->addAction(windowLayersAction);
        menu_Docks->addAction(windowInfoAction);
        menu_Docks->addAction(windowDirtyAction);
        menu_Docks->addAction(windowGPSAction);
        menu_Docks->addAction(windowGeoimageAction);
        menu_Docks->addAction(windowStylesAction);
        menu_Docks->addAction(windowFeatsAction);
        menu_Feature->addAction(featureSelectChildrenAction);
        menu_Feature->addAction(featureSelectParentsAction);
        menu_Feature->addSeparator();
        menu_Feature->addAction(featureDeleteAction);
        menu_Feature->addAction(featureCommitAction);
        menu_Feature->addSeparator();
        menu_Feature->addAction(featureDownloadMissingChildrenAction);
        menuLayers->addAction(layersNewImageAction);
        menuLayers->addAction(layersNewDrawingAction);
        menuLayers->addAction(layersNewFilterAction);
        menuLayers->addAction(layersMapdustAction);
        menuLayers->addSeparator();
        menuGps->addAction(gpsConnectAction);
        menuGps->addAction(gpsReplayAction);
        menuGps->addSeparator();
        menuGps->addAction(gpsRecordAction);
        menuGps->addAction(gpsPauseAction);
        menuGps->addAction(gpsDisconnectAction);
        menuGps->addSeparator();
        menuGps->addAction(gpsCenterAction);
        menuRelation->addAction(relationAddMemberAction);
        menuRelation->addAction(relationRemoveMemberAction);
        menuRelation->addAction(relationAddToMultipolygonAction);
        menu_Show->addAction(viewDownloadedAction);
        menu_Show->addAction(viewDirtyAction);
        menu_Show->addAction(menuShow_directional_Arrows->menuAction());
        menu_Show->addSeparator();
        menu_Show->addAction(viewStyleForegroundAction);
        menu_Show->addAction(viewStyleBackgroundAction);
        menu_Show->addAction(viewStyleTouchupAction);
        menu_Show->addAction(viewNamesAction);
        menu_Show->addSeparator();
        menu_Show->addAction(viewTrackPointsAction);
        menu_Show->addAction(viewVirtualNodesAction);
        menu_Show->addAction(viewTrackSegmentsAction);
        menu_Show->addAction(viewRelationsAction);
        menu_Show->addSeparator();
        menu_Show->addAction(viewPhotosAction);
        menu_Show->addAction(viewScaleAction);
        menu_Show->addAction(viewShowLatLonGridAction);
        menuShow_directional_Arrows->addAction(viewArrowsNeverAction);
        menuShow_directional_Arrows->addAction(viewArrowsOnewayAction);
        menuShow_directional_Arrows->addAction(viewArrowsAlwaysAction);
        toolBar->addAction(fileDownloadAction);
        toolBar->addAction(fileDownloadMoreAction);
        toolBar->addAction(fileUploadAction);
        toolBar->addAction(fileSaveAction);
        toolBar->addSeparator();
        toolBar->addAction(editCopyAction);
        toolBar->addAction(editPasteFeatureAction);
        toolBar->addAction(editPasteMergeAction);
        toolBar->addAction(editRemoveAction);
        toolBar->addSeparator();
        toolBar->addAction(editUndoAction);
        toolBar->addAction(editRedoAction);
        toolBar->addSeparator();
        toolBar->addAction(editPropertiesAction);
        toolBar->addAction(editMoveAction);
        toolBar->addAction(editRotateAction);
        toolBar->addAction(editScaleAction);
        toolBar->addAction(createNodeAction);
        toolBar->addAction(createRoadAction);
        toolBar->addAction(createAreaAction);
        toolBar->addSeparator();
        toolBar->addAction(nodeAlignAction);
        toolBar->addAction(nodeSpreadAction);
        toolBar->addAction(nodeDetachAction);
        toolBar->addAction(roadSplitAction);
        toolBar->addAction(roadBreakAction);
        toolBar->addAction(roadJoinAction);
        toolBar->addAction(editReverseAction);
        toolBar->addAction(roadSubdivideAction);
        toolBar->addAction(areaJoinAction);
        toolBar->addAction(areaSplitAction);
        toolBar->addAction(areaTerraceAction);
        toolBar->addAction(roadAxisAlignAction);
        toolBar->addSeparator();
        toolBar->addAction(markBridgeAction);

        retranslateUi(MainWindow);
        QObject::connect(fileQuitAction, &QAction::triggered, MainWindow, qOverload<>(&QMainWindow::close));

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Merkaartor", nullptr));
        fileQuitAction->setText(QCoreApplication::translate("MainWindow", "&Quit", nullptr));
        helpAboutAction->setText(QCoreApplication::translate("MainWindow", "&About", nullptr));
        fileOpenAction->setText(QCoreApplication::translate("MainWindow", "&Open", nullptr));
#if QT_CONFIG(statustip)
        fileOpenAction->setStatusTip(QCoreApplication::translate("MainWindow", "Create a new document and import a file", nullptr));
#endif // QT_CONFIG(statustip)
        viewZoomAllAction->setText(QCoreApplication::translate("MainWindow", "Zoom &all", nullptr));
        viewZoomWindowAction->setText(QCoreApplication::translate("MainWindow", "Zoom &window", nullptr));
        viewZoomWindowAction->setIconText(QCoreApplication::translate("MainWindow", "Zoom window", nullptr));
#if QT_CONFIG(tooltip)
        viewZoomWindowAction->setToolTip(QCoreApplication::translate("MainWindow", "Zoom window", nullptr));
#endif // QT_CONFIG(tooltip)
        viewZoomOutAction->setText(QCoreApplication::translate("MainWindow", "Zoom &out", nullptr));
        viewZoomInAction->setText(QCoreApplication::translate("MainWindow", "Zoom &in", nullptr));
        viewZoomInAction->setIconText(QCoreApplication::translate("MainWindow", "Zoom in", nullptr));
#if QT_CONFIG(tooltip)
        viewZoomInAction->setToolTip(QCoreApplication::translate("MainWindow", "Zoom in", nullptr));
#endif // QT_CONFIG(tooltip)
        createWayAction->setText(QCoreApplication::translate("MainWindow", "Curved link", nullptr));
        createWayAction->setIconText(QCoreApplication::translate("MainWindow", "Curved link", nullptr));
#if QT_CONFIG(tooltip)
        createWayAction->setToolTip(QCoreApplication::translate("MainWindow", "Curved link", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        createWayAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        editUndoAction->setText(QCoreApplication::translate("MainWindow", "&Undo", nullptr));
        editRedoAction->setText(QCoreApplication::translate("MainWindow", "&Redo", nullptr));
        editMoveAction->setText(QCoreApplication::translate("MainWindow", "&Move", nullptr));
        fileImportGDALAction->setText(QCoreApplication::translate("MainWindow", "Import using &GDAL", nullptr));
#if QT_CONFIG(statustip)
        fileImportGDALAction->setStatusTip(QCoreApplication::translate("MainWindow", "Import a file into the current document using GDAL", nullptr));
#endif // QT_CONFIG(statustip)
        fileImportAction->setText(QCoreApplication::translate("MainWindow", "&Import", nullptr));
#if QT_CONFIG(statustip)
        fileImportAction->setStatusTip(QCoreApplication::translate("MainWindow", "Import a file into the current document", nullptr));
#endif // QT_CONFIG(statustip)
        fileDownloadAction->setText(QCoreApplication::translate("MainWindow", "&Download", nullptr));
#if QT_CONFIG(tooltip)
        fileDownloadAction->setToolTip(QCoreApplication::translate("MainWindow", "Download map data for a new area", nullptr));
#endif // QT_CONFIG(tooltip)
        createLinearWayAction->setText(QCoreApplication::translate("MainWindow", "Link", nullptr));
        createLinearWayAction->setIconText(QCoreApplication::translate("MainWindow", "Create link", nullptr));
#if QT_CONFIG(tooltip)
        createLinearWayAction->setToolTip(QCoreApplication::translate("MainWindow", "Create link", nullptr));
#endif // QT_CONFIG(tooltip)
        editPropertiesAction->setText(QCoreApplication::translate("MainWindow", "&Select", nullptr));
        fileUploadAction->setText(QCoreApplication::translate("MainWindow", "&Upload", nullptr));
#if QT_CONFIG(tooltip)
        fileUploadAction->setToolTip(QCoreApplication::translate("MainWindow", "Upload changes to the server", nullptr));
#endif // QT_CONFIG(tooltip)
        editRemoveAction->setText(QCoreApplication::translate("MainWindow", "R&emove", nullptr));
#if QT_CONFIG(tooltip)
        editRemoveAction->setToolTip(QCoreApplication::translate("MainWindow", "Remove selected features", nullptr));
#endif // QT_CONFIG(tooltip)
        createRoadAction->setText(QCoreApplication::translate("MainWindow", "&Way", nullptr));
#if QT_CONFIG(tooltip)
        createRoadAction->setToolTip(QCoreApplication::translate("MainWindow", "Create new way", nullptr));
#endif // QT_CONFIG(tooltip)
        createNodeAction->setText(QCoreApplication::translate("MainWindow", "&Node", nullptr));
#if QT_CONFIG(tooltip)
        createNodeAction->setToolTip(QCoreApplication::translate("MainWindow", "Create new node", nullptr));
#endif // QT_CONFIG(tooltip)
        editReverseAction->setText(QCoreApplication::translate("MainWindow", "&Reverse", nullptr));
#if QT_CONFIG(tooltip)
        editReverseAction->setToolTip(QCoreApplication::translate("MainWindow", "Reverse way direction", nullptr));
#endif // QT_CONFIG(tooltip)
        viewGotoAction->setText(QCoreApplication::translate("MainWindow", "&Go to\342\200\246", nullptr));
        createDoubleWayAction->setText(QCoreApplication::translate("MainWindow", "&Parallel way", nullptr));
#if QT_CONFIG(tooltip)
        createDoubleWayAction->setToolTip(QCoreApplication::translate("MainWindow", "Create a parallel way", nullptr));
#endif // QT_CONFIG(tooltip)
        createRoundaboutAction->setText(QCoreApplication::translate("MainWindow", "R&oundabout", nullptr));
#if QT_CONFIG(tooltip)
        createRoundaboutAction->setToolTip(QCoreApplication::translate("MainWindow", "Create Roundabout", nullptr));
#endif // QT_CONFIG(tooltip)
        fileNewAction->setText(QCoreApplication::translate("MainWindow", "&New", nullptr));
#if QT_CONFIG(statustip)
        fileNewAction->setStatusTip(QCoreApplication::translate("MainWindow", "New document", nullptr));
#endif // QT_CONFIG(statustip)
        roadSplitAction->setText(QCoreApplication::translate("MainWindow", "&Split", nullptr));
#if QT_CONFIG(tooltip)
        roadSplitAction->setToolTip(QCoreApplication::translate("MainWindow", "Split way into separate (connected) ways", nullptr));
#endif // QT_CONFIG(tooltip)
        roadJoinAction->setText(QCoreApplication::translate("MainWindow", "&Join", nullptr));
#if QT_CONFIG(tooltip)
        roadJoinAction->setToolTip(QCoreApplication::translate("MainWindow", "Join connected ways into a single way", nullptr));
#endif // QT_CONFIG(tooltip)
        roadBreakAction->setText(QCoreApplication::translate("MainWindow", "&Break apart", nullptr));
        roadBreakAction->setIconText(QCoreApplication::translate("MainWindow", "Break", nullptr));
#if QT_CONFIG(tooltip)
        roadBreakAction->setToolTip(QCoreApplication::translate("MainWindow", "Break apart connected ways", nullptr));
#endif // QT_CONFIG(tooltip)
        createRelationAction->setText(QCoreApplication::translate("MainWindow", "Re&lation", nullptr));
#if QT_CONFIG(tooltip)
        createRelationAction->setToolTip(QCoreApplication::translate("MainWindow", "Create Relation", nullptr));
#endif // QT_CONFIG(tooltip)
        createAreaAction->setText(QCoreApplication::translate("MainWindow", "&Area", nullptr));
#if QT_CONFIG(tooltip)
        createAreaAction->setToolTip(QCoreApplication::translate("MainWindow", "Create new area", nullptr));
#endif // QT_CONFIG(tooltip)
        editMapStyleAction->setText(QCoreApplication::translate("MainWindow", "&Edit\342\200\246", nullptr));
        mapStyleSaveAsAction->setText(QCoreApplication::translate("MainWindow", "Save &as\342\200\246", nullptr));
        mapStyleLoadAction->setText(QCoreApplication::translate("MainWindow", "&Load\342\200\246", nullptr));
        createCurvedRoadAction->setText(QCoreApplication::translate("MainWindow", "&Curved way", nullptr));
        toolsPreferencesAction->setText(QCoreApplication::translate("MainWindow", "&Preferences\342\200\246", nullptr));
        exportOSMAllAction->setText(QCoreApplication::translate("MainWindow", "&All\342\200\246", nullptr));
#if QT_CONFIG(statustip)
        exportOSMAllAction->setStatusTip(QCoreApplication::translate("MainWindow", "Export all visible layers to a file", nullptr));
#endif // QT_CONFIG(statustip)
        exportOSMBinAllAction->setText(QCoreApplication::translate("MainWindow", "&All\342\200\246", nullptr));
#if QT_CONFIG(statustip)
        exportOSMBinAllAction->setStatusTip(QCoreApplication::translate("MainWindow", "Export all visible layers to a file", nullptr));
#endif // QT_CONFIG(statustip)
        editSelectAction->setText(QCoreApplication::translate("MainWindow", "&Find\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        editSelectAction->setToolTip(QCoreApplication::translate("MainWindow", "Find", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editSelectAction->setStatusTip(QCoreApplication::translate("MainWindow", "Find and select items", nullptr));
#endif // QT_CONFIG(statustip)
        exportOSMViewportAction->setText(QCoreApplication::translate("MainWindow", "&Viewport\342\200\246", nullptr));
#if QT_CONFIG(statustip)
        exportOSMViewportAction->setStatusTip(QCoreApplication::translate("MainWindow", "Export the features in the viewport to a file", nullptr));
#endif // QT_CONFIG(statustip)
        exportOSMBinViewportAction->setText(QCoreApplication::translate("MainWindow", "&Viewport\342\200\246", nullptr));
#if QT_CONFIG(statustip)
        exportOSMBinViewportAction->setStatusTip(QCoreApplication::translate("MainWindow", "Export the features in the viewport to a file", nullptr));
#endif // QT_CONFIG(statustip)
        bookmarkAddAction->setText(QCoreApplication::translate("MainWindow", "&Add\342\200\246", nullptr));
        bookmarkRemoveAction->setText(QCoreApplication::translate("MainWindow", "&Remove\342\200\246", nullptr));
        nodeMergeAction->setText(QCoreApplication::translate("MainWindow", "&Merge", nullptr));
#if QT_CONFIG(tooltip)
        nodeMergeAction->setToolTip(QCoreApplication::translate("MainWindow", "Node Merge", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        nodeMergeAction->setStatusTip(QCoreApplication::translate("MainWindow", "Merge the selected nodes (first selected will remain)", nullptr));
#endif // QT_CONFIG(statustip)
        fileSaveAsAction->setText(QCoreApplication::translate("MainWindow", "Save &as\342\200\246", nullptr));
        fileSaveAction->setText(QCoreApplication::translate("MainWindow", "&Save", nullptr));
#if QT_CONFIG(tooltip)
        fileSaveAction->setToolTip(QCoreApplication::translate("MainWindow", "Save as file", nullptr));
#endif // QT_CONFIG(tooltip)
        fileDownloadMoreAction->setText(QCoreApplication::translate("MainWindow", "Download more", nullptr));
#if QT_CONFIG(tooltip)
        fileDownloadMoreAction->setToolTip(QCoreApplication::translate("MainWindow", "Download more map data for the current area", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileDownloadMoreAction->setStatusTip(QCoreApplication::translate("MainWindow", "Download the current view into the previous download layer", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        fileDownloadMoreAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Download the current view into the previous download layer", nullptr));
#endif // QT_CONFIG(whatsthis)
        action_Docks->setText(QCoreApplication::translate("MainWindow", "&Docks", nullptr));
        windowPropertiesAction->setText(QCoreApplication::translate("MainWindow", "&Properties", nullptr));
#if QT_CONFIG(tooltip)
        windowPropertiesAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show property dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowPropertiesAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/Show the property dock", nullptr));
#endif // QT_CONFIG(statustip)
        windowLayersAction->setText(QCoreApplication::translate("MainWindow", "&Layers", nullptr));
#if QT_CONFIG(tooltip)
        windowLayersAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show layer dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowLayersAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/show layer dock", nullptr));
#endif // QT_CONFIG(statustip)
        windowInfoAction->setText(QCoreApplication::translate("MainWindow", "&Info", nullptr));
#if QT_CONFIG(tooltip)
        windowInfoAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show info dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowInfoAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/show info dock", nullptr));
#endif // QT_CONFIG(statustip)
        nodeAlignAction->setText(QCoreApplication::translate("MainWindow", "&Align", nullptr));
#if QT_CONFIG(tooltip)
        nodeAlignAction->setToolTip(QCoreApplication::translate("MainWindow", "Align nodes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        nodeAlignAction->setStatusTip(QCoreApplication::translate("MainWindow", "Align selected nodes. First two selected give the line.", nullptr));
#endif // QT_CONFIG(statustip)
        nodeSpreadAction->setText(QCoreApplication::translate("MainWindow", "&Spread", nullptr));
#if QT_CONFIG(tooltip)
        nodeSpreadAction->setToolTip(QCoreApplication::translate("MainWindow", "Spread nodes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        nodeSpreadAction->setStatusTip(QCoreApplication::translate("MainWindow", "Align and spread selected nodes equally.", nullptr));
#endif // QT_CONFIG(statustip)
        windowDirtyAction->setText(QCoreApplication::translate("MainWindow", "&Undo", nullptr));
#if QT_CONFIG(tooltip)
        windowDirtyAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show undo dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowDirtyAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/show undo dock", nullptr));
#endif // QT_CONFIG(statustip)
        viewDownloadedAction->setText(QCoreApplication::translate("MainWindow", "&Downloaded areas", nullptr));
        editCopyAction->setText(QCoreApplication::translate("MainWindow", "&Copy", nullptr));
#if QT_CONFIG(tooltip)
        editCopyAction->setToolTip(QCoreApplication::translate("MainWindow", "Copy selected features and tags to the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editCopyAction->setStatusTip(QCoreApplication::translate("MainWindow", "Copy the selected feature's tags to the clipboard; if the feature is a trackpoint, copy the coordinates, too.", nullptr));
#endif // QT_CONFIG(statustip)
        editPasteOverwriteAction->setText(QCoreApplication::translate("MainWindow", "Paste Tags (&Overwrite)", nullptr));
#if QT_CONFIG(statustip)
        editPasteOverwriteAction->setStatusTip(QCoreApplication::translate("MainWindow", "Paste (and overwrite) the tags in the clipboard to the selected feature.", nullptr));
#endif // QT_CONFIG(statustip)
        editPasteMergeAction->setText(QCoreApplication::translate("MainWindow", "Paste Tags (&Merge)", nullptr));
        editPasteMergeAction->setIconText(QCoreApplication::translate("MainWindow", "Paste tags", nullptr));
#if QT_CONFIG(tooltip)
        editPasteMergeAction->setToolTip(QCoreApplication::translate("MainWindow", "Paste tags from the clipboard (Merge with existing tags)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editPasteMergeAction->setStatusTip(QCoreApplication::translate("MainWindow", "Merge the tags in the clipboard with the ones of the selected feature.", nullptr));
#endif // QT_CONFIG(statustip)
        exportOSMSelectedAction->setText(QCoreApplication::translate("MainWindow", "Selected\342\200\246", nullptr));
        exportOSMBinSelectedAction->setText(QCoreApplication::translate("MainWindow", "Selected\342\200\246", nullptr));
        editPasteFeatureAction->setText(QCoreApplication::translate("MainWindow", "Paste feature(s)", nullptr));
        editPasteFeatureAction->setIconText(QCoreApplication::translate("MainWindow", "Paste", nullptr));
#if QT_CONFIG(tooltip)
        editPasteFeatureAction->setToolTip(QCoreApplication::translate("MainWindow", "Paste features from the clipboard", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editPasteFeatureAction->setStatusTip(QCoreApplication::translate("MainWindow", "Paste the features in the clipboard; If the features'id are already in the document, overwrite them.", nullptr));
#endif // QT_CONFIG(statustip)
        exportOSMAction->setText(QCoreApplication::translate("MainWindow", "OSM (XML)", nullptr));
        exportOSMBinAction->setText(QCoreApplication::translate("MainWindow", "OSM (binary)", nullptr));
        featureCommitAction->setText(QCoreApplication::translate("MainWindow", "&Force upload", nullptr));
#if QT_CONFIG(tooltip)
        featureCommitAction->setToolTip(QCoreApplication::translate("MainWindow", "Commit feature to the dirty layer", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        featureCommitAction->setStatusTip(QCoreApplication::translate("MainWindow", "Commit the selected feature from a non-uploadable layer (e.g. \\\"Track\\\" or \\\"Extract\\\") to the dirty layer, ready for upload.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(whatsthis)
        featureCommitAction->setWhatsThis(QCoreApplication::translate("MainWindow", "Commit the selected feature from a non-uploadable layer (e.g. \\\"Track\\\" or \\\"Extract\\\") to the dirty layer, ready for upload.", nullptr));
#endif // QT_CONFIG(whatsthis)
        exportGPXAction->setText(QCoreApplication::translate("MainWindow", "GPX", nullptr));
        exportKMLAction->setText(QCoreApplication::translate("MainWindow", "KML", nullptr));
        windowToolbarAction->setText(QCoreApplication::translate("MainWindow", "Toggle toolbar", nullptr));
#if QT_CONFIG(tooltip)
        windowToolbarAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show Toolbar", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowToolbarAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/Show toolbar", nullptr));
#endif // QT_CONFIG(statustip)
        windowHideAllAction->setText(QCoreApplication::translate("MainWindow", "Hide all", nullptr));
        windowShowAllAction->setText(QCoreApplication::translate("MainWindow", "Show all", nullptr));
        layersAddImageAction->setText(QCoreApplication::translate("MainWindow", "&Image layer", nullptr));
        renderNativeAction->setText(QCoreApplication::translate("MainWindow", "&Raster/SVG", nullptr));
        viewTrackPointsAction->setText(QCoreApplication::translate("MainWindow", "&Nodes", nullptr));
        viewNamesAction->setText(QCoreApplication::translate("MainWindow", "Na&mes", nullptr));
        gpsConnectAction->setText(QCoreApplication::translate("MainWindow", "&Start", nullptr));
#if QT_CONFIG(tooltip)
        gpsConnectAction->setToolTip(QCoreApplication::translate("MainWindow", "Start GPS", nullptr));
#endif // QT_CONFIG(tooltip)
        gpsReplayAction->setText(QCoreApplication::translate("MainWindow", "&Replay\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        gpsReplayAction->setToolTip(QCoreApplication::translate("MainWindow", "Replay GPS", nullptr));
#endif // QT_CONFIG(tooltip)
        windowGPSAction->setText(QCoreApplication::translate("MainWindow", "&GPS", nullptr));
#if QT_CONFIG(tooltip)
        windowGPSAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show GPS dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowGPSAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/show GPS dock", nullptr));
#endif // QT_CONFIG(statustip)
        gpsDisconnectAction->setText(QCoreApplication::translate("MainWindow", "S&top", nullptr));
#if QT_CONFIG(tooltip)
        gpsDisconnectAction->setToolTip(QCoreApplication::translate("MainWindow", "Stop GPS", nullptr));
#endif // QT_CONFIG(tooltip)
        gpsCenterAction->setText(QCoreApplication::translate("MainWindow", "&Center on GPS", nullptr));
        viewTrackSegmentsAction->setText(QCoreApplication::translate("MainWindow", "Track &segments", nullptr));
        viewScaleAction->setText(QCoreApplication::translate("MainWindow", "&Scale", nullptr));
        viewRelationsAction->setText(QCoreApplication::translate("MainWindow", "&Relations", nullptr));
        viewStyleForegroundAction->setText(QCoreApplication::translate("MainWindow", "Show road backgrounds", nullptr));
        viewStyleBackgroundAction->setText(QCoreApplication::translate("MainWindow", "Show road boundaries", nullptr));
        viewStyleTouchupAction->setText(QCoreApplication::translate("MainWindow", "Show touchup", nullptr));
        gpsRecordAction->setText(QCoreApplication::translate("MainWindow", "Record", nullptr));
        gpsRecordAction->setIconText(QCoreApplication::translate("MainWindow", "Record", nullptr));
#if QT_CONFIG(tooltip)
        gpsRecordAction->setToolTip(QCoreApplication::translate("MainWindow", "Record GPS", nullptr));
#endif // QT_CONFIG(tooltip)
        gpsPauseAction->setText(QCoreApplication::translate("MainWindow", "Pause", nullptr));
#if QT_CONFIG(tooltip)
        gpsPauseAction->setToolTip(QCoreApplication::translate("MainWindow", "Pause GPS", nullptr));
#endif // QT_CONFIG(tooltip)
        windowGeoimageAction->setText(QCoreApplication::translate("MainWindow", "G&eoImage", nullptr));
#if QT_CONFIG(tooltip)
        windowGeoimageAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show GeoImage dock", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        windowGeoimageAction->setStatusTip(QCoreApplication::translate("MainWindow", "Hide/show GeoImage dock", nullptr));
#endif // QT_CONFIG(statustip)
        toolsWorldOsbAction->setText(QCoreApplication::translate("MainWindow", "World OSB manager\342\200\246", nullptr));
        toolsShortcutsAction->setText(QCoreApplication::translate("MainWindow", "&Shortcut editor\342\200\246", nullptr));
        toolTemplatesLoadAction->setText(QCoreApplication::translate("MainWindow", "&Load\342\200\246", nullptr));
        toolTemplatesMergeAction->setText(QCoreApplication::translate("MainWindow", "&Merge\342\200\246", nullptr));
        toolTemplatesSaveAction->setText(QCoreApplication::translate("MainWindow", "&Save\342\200\246", nullptr));
        relationAddMemberAction->setText(QCoreApplication::translate("MainWindow", "&Add member", nullptr));
        relationRemoveMemberAction->setText(QCoreApplication::translate("MainWindow", "&Remove member", nullptr));
        viewArrowsNeverAction->setText(QCoreApplication::translate("MainWindow", "&Never", nullptr));
        viewArrowsOnewayAction->setText(QCoreApplication::translate("MainWindow", "For 'oneway' ways", nullptr));
        viewArrowsAlwaysAction->setText(QCoreApplication::translate("MainWindow", "&Always", nullptr));
        nodeDetachAction->setText(QCoreApplication::translate("MainWindow", "&Detach", nullptr));
#if QT_CONFIG(tooltip)
        nodeDetachAction->setToolTip(QCoreApplication::translate("MainWindow", "Detach node from a way", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        nodeDetachAction->setStatusTip(QCoreApplication::translate("MainWindow", "Detach a node from a way", nullptr));
#endif // QT_CONFIG(statustip)
        fileWorkOfflineAction->setText(QCoreApplication::translate("MainWindow", "&Work offline", nullptr));
        renderSVGAction->setText(QCoreApplication::translate("MainWindow", "SVG", nullptr));
        windowStylesAction->setText(QCoreApplication::translate("MainWindow", "&Styles", nullptr));
        windowStylesAction->setIconText(QCoreApplication::translate("MainWindow", "Hide/show style dock", nullptr));
#if QT_CONFIG(tooltip)
        windowStylesAction->setToolTip(QCoreApplication::translate("MainWindow", "Hide/show style dock", nullptr));
#endif // QT_CONFIG(tooltip)
        toolsWMSServersAction->setText(QCoreApplication::translate("MainWindow", "&WMS servers editor\342\200\246", nullptr));
        toolsTMSServersAction->setText(QCoreApplication::translate("MainWindow", "&TMS servers editor\342\200\246", nullptr));
        toolsResetDiscardableAction->setText(QCoreApplication::translate("MainWindow", "&Reset discardable dialog status", nullptr));
        gpsPopupAction->setText(QCoreApplication::translate("MainWindow", "GPS menu", nullptr));
        cameraAction->setText(QCoreApplication::translate("MainWindow", "Camera", nullptr));
        roadCreateJunctionAction->setText(QCoreApplication::translate("MainWindow", "Create &junction", nullptr));
        editRotateAction->setText(QCoreApplication::translate("MainWindow", "Rotate", nullptr));
        createPolygonAction->setText(QCoreApplication::translate("MainWindow", "&Polygon", nullptr));
        createRectangleAction->setText(QCoreApplication::translate("MainWindow", "Equiangular &building", nullptr));
        layersNewImageAction->setText(QCoreApplication::translate("MainWindow", "Add new &image layer", nullptr));
        windowFeatsAction->setText(QCoreApplication::translate("MainWindow", "Features", nullptr));
        roadAddStreetNumbersAction->setText(QCoreApplication::translate("MainWindow", "Add street &numbers (Karlsruhe schema)", nullptr));
        roadSubdivideAction->setText(QCoreApplication::translate("MainWindow", "&Subdivide", nullptr));
#if QT_CONFIG(tooltip)
        roadSubdivideAction->setToolTip(QCoreApplication::translate("MainWindow", "Subdivide segment equally", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        roadSubdivideAction->setStatusTip(QCoreApplication::translate("MainWindow", "Subdivide a selected way segment (the way and two adjacent nodes) into equidistant segments.", nullptr));
#endif // QT_CONFIG(statustip)
        viewVirtualNodesAction->setText(QCoreApplication::translate("MainWindow", "&Virtual nodes", nullptr));
        viewShowLatLonGridAction->setText(QCoreApplication::translate("MainWindow", "Lat/Lon &grid", nullptr));
        viewLockZoomAction->setText(QCoreApplication::translate("MainWindow", "&Lock zoom to tiled background", nullptr));
        toolsProjectionsAction->setText(QCoreApplication::translate("MainWindow", "&Projection editor\342\200\246", nullptr));
        viewPhotosAction->setText(QCoreApplication::translate("MainWindow", "&Photos on map", nullptr));
        exportOSCAction->setText(QCoreApplication::translate("MainWindow", "OsmChange (OSC)", nullptr));
        featureDeleteAction->setText(QCoreApplication::translate("MainWindow", "Force delete", nullptr));
        layersOpenstreetbugsAction->setText(QCoreApplication::translate("MainWindow", "Add new OpenStreet&Bugs layer", nullptr));
        featureOsbClose->setText(QCoreApplication::translate("MainWindow", "Close", nullptr));
        roadSimplifyAction->setText(QCoreApplication::translate("MainWindow", "S&implify", nullptr));
#if QT_CONFIG(tooltip)
        roadSimplifyAction->setToolTip(QCoreApplication::translate("MainWindow", "Simplify road(s)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        roadSimplifyAction->setStatusTip(QCoreApplication::translate("MainWindow", "Simplify way by removing unnecessary child nodes", nullptr));
#endif // QT_CONFIG(statustip)
        toolsFiltersAction->setText(QCoreApplication::translate("MainWindow", "&Filter editor\342\200\246", nullptr));
        filterNoneAction->setText(QCoreApplication::translate("MainWindow", "&None", nullptr));
        areaJoinAction->setText(QCoreApplication::translate("MainWindow", "&Join areas", nullptr));
#if QT_CONFIG(tooltip)
        areaJoinAction->setToolTip(QCoreApplication::translate("MainWindow", "Join adjacent areas", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        areaJoinAction->setStatusTip(QCoreApplication::translate("MainWindow", "Join areas that touch each-other.", nullptr));
#endif // QT_CONFIG(statustip)
        areaSplitAction->setText(QCoreApplication::translate("MainWindow", "&Split area", nullptr));
#if QT_CONFIG(tooltip)
        areaSplitAction->setToolTip(QCoreApplication::translate("MainWindow", "Split area between two nodes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        areaSplitAction->setStatusTip(QCoreApplication::translate("MainWindow", "Split selected area between two selected nodes into two separate areas.", nullptr));
#endif // QT_CONFIG(statustip)
        areaTerraceAction->setText(QCoreApplication::translate("MainWindow", "&Terrace", nullptr));
#if QT_CONFIG(tooltip)
        areaTerraceAction->setToolTip(QCoreApplication::translate("MainWindow", "Terrace area into residences", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        areaTerraceAction->setStatusTip(QCoreApplication::translate("MainWindow", "Split a selected area into terraced residences.", nullptr));
#endif // QT_CONFIG(statustip)
        toolsToolbarsAction->setText(QCoreApplication::translate("MainWindow", "Toolbar editor\342\200\246", nullptr));
        roadAxisAlignAction->setText(QCoreApplication::translate("MainWindow", "A&xis Align", nullptr));
#if QT_CONFIG(tooltip)
        roadAxisAlignAction->setToolTip(QCoreApplication::translate("MainWindow", "Align edges to regular axes", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        roadAxisAlignAction->setStatusTip(QCoreApplication::translate("MainWindow", "Align edges to a certain number of regularly spaced axis.", nullptr));
#endif // QT_CONFIG(statustip)
        filePrintAction->setText(QCoreApplication::translate("MainWindow", "&Print\342\200\246", nullptr));
        filePrintPreviewAction->setText(QCoreApplication::translate("MainWindow", "Print preview\342\200\246", nullptr));
        filePropertiesAction->setText(QCoreApplication::translate("MainWindow", "Properties\342\200\246", nullptr));
        viewDirtyAction->setText(QCoreApplication::translate("MainWindow", "Highlight dirt&y features", nullptr));
        layersNewDrawingAction->setText(QCoreApplication::translate("MainWindow", "Add new &drawing layer", nullptr));
        editCutAction->setText(QCoreApplication::translate("MainWindow", "Cu&t", nullptr));
        layersNewFilterAction->setText(QCoreApplication::translate("MainWindow", "Add new &filter layer", nullptr));
        roadExtrudeAction->setText(QCoreApplication::translate("MainWindow", "E&xtrude", nullptr));
#if QT_CONFIG(tooltip)
        roadExtrudeAction->setToolTip(QCoreApplication::translate("MainWindow", "Extrude interaction for ways (JOSM style)", nullptr));
#endif // QT_CONFIG(tooltip)
        featureSelectAction->setText(QCoreApplication::translate("MainWindow", "Select toggle", nullptr));
        featureSelectChildrenAction->setText(QCoreApplication::translate("MainWindow", "Include children in selection", nullptr));
        editScaleAction->setText(QCoreApplication::translate("MainWindow", "Scale", nullptr));
        fileSaveAsTemplateAction->setText(QCoreApplication::translate("MainWindow", "Save as template document\342\200\246", nullptr));
        actionCreate_Multipolygon->setText(QCoreApplication::translate("MainWindow", "Create multipolygon", nullptr));
        relationAddToMultipolygonAction->setText(QCoreApplication::translate("MainWindow", "Add to multi&polygon", nullptr));
        mapStyleSaveAction->setText(QCoreApplication::translate("MainWindow", "&Save", nullptr));
        exportGDALAction->setText(QCoreApplication::translate("MainWindow", "GDAL SQLite/SpatiLite", nullptr));
        roadBingExtractAction->setText(QCoreApplication::translate("MainWindow", "Bing road detector", nullptr));
        toolsRebuildHistoryAction->setText(QCoreApplication::translate("MainWindow", "Rebuild &history", nullptr));
        layersMapdustAction->setText(QCoreApplication::translate("MainWindow", "Add new Map&dust layer", nullptr));
        viewWireframeAction->setText(QCoreApplication::translate("MainWindow", "Wireframe", nullptr));
        featureSelectParentsAction->setText(QCoreApplication::translate("MainWindow", "Select parent(s)", nullptr));
        featureDownloadMissingChildrenAction->setText(QCoreApplication::translate("MainWindow", "Download missing children", nullptr));
        markBridgeAction->setText(QCoreApplication::translate("MainWindow", "&Bridge", nullptr));
#if QT_CONFIG(tooltip)
        markBridgeAction->setToolTip(QCoreApplication::translate("MainWindow", "<html><head/><body><p>Transform way to a bridge</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "&Help", nullptr));
        menuCreate->setTitle(QCoreApplication::translate("MainWindow", "&Create", nullptr));
        menuRoad->setTitle(QCoreApplication::translate("MainWindow", "Wa&y", nullptr));
        menuEdit->setTitle(QCoreApplication::translate("MainWindow", "&Edit", nullptr));
        menuView->setTitle(QCoreApplication::translate("MainWindow", "&View", nullptr));
        menuBookmarks->setTitle(QCoreApplication::translate("MainWindow", "&Bookmarks", nullptr));
        mnuProjections->setTitle(QCoreApplication::translate("MainWindow", "&Projection", nullptr));
        mnuAreaOpacity->setTitle(QCoreApplication::translate("MainWindow", "Area &opacity", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "&File", nullptr));
        menuExport->setTitle(QCoreApplication::translate("MainWindow", "&Export", nullptr));
        menuRecentOpen->setTitle(QCoreApplication::translate("MainWindow", "Re&cently opened", nullptr));
        menuRecentImport->setTitle(QCoreApplication::translate("MainWindow", "Recen&tly imported", nullptr));
        menuTools->setTitle(QCoreApplication::translate("MainWindow", "T&ools", nullptr));
        menuStyles->setTitle(QCoreApplication::translate("MainWindow", "&Style", nullptr));
        designerMenu->setTitle(QCoreApplication::translate("MainWindow", "Ta&g templates", nullptr));
        menu_Node->setTitle(QCoreApplication::translate("MainWindow", "&Node", nullptr));
        menuWindow->setTitle(QCoreApplication::translate("MainWindow", "&Window", nullptr));
        menu_Docks->setTitle(QCoreApplication::translate("MainWindow", "&Docks", nullptr));
        menu_Feature->setTitle(QCoreApplication::translate("MainWindow", "Fea&ture", nullptr));
        menuLayers->setTitle(QCoreApplication::translate("MainWindow", "&Layers", nullptr));
        menuGps->setTitle(QCoreApplication::translate("MainWindow", "&GPS", nullptr));
        menuRelation->setTitle(QCoreApplication::translate("MainWindow", "Rel&ation", nullptr));
        menu_Show->setTitle(QCoreApplication::translate("MainWindow", "S&how", nullptr));
        menuShow_directional_Arrows->setTitle(QCoreApplication::translate("MainWindow", "Directional &arrows", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("MainWindow", "Main toolbar", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
